/*++

Copyright (c) 1993  Microsoft Corporation

Module Name:

    parallel.h

Abstract:

    This file defines the services supplied by the parallel port driver.

Author:

    norbertk

Revision History:

--*/

#ifndef _PARALLEL_
#define _PARALLEL_

#include <ntddpar.h>

//
// Interface GUID
//
#ifdef WANT_WDM

DEFINE_GUID(GUID_PARALLEL_DEVICE, 0x97F76EF0, 0xF883, 0x11D0, 0xAF, 0x1F, 0x00, 0x00, 0xF8, 0x00, 0x84, 0x5C);
DEFINE_GUID(GUID_PARCLASS_DEVICE, 0x811FC6A5, 0xF728, 0x11D0, 0xA5, 0x37, 0x00, 0x00, 0xF8, 0x75, 0x3E, 0xD1);

#endif

//
// Define the parallel port device name strings.
//

#define DD_PARALLEL_PORT_BASE_NAME_U   L"ParallelPort"

//
// NtDeviceIoControlFile internal IoControlCode values for parallel device.
//

#define IOCTL_INTERNAL_PARALLEL_PORT_ALLOCATE               CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 11, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_GET_PARALLEL_PORT_INFO               CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 12, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_PARALLEL_CONNECT_INTERRUPT           CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 13, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_PARALLEL_DISCONNECT_INTERRUPT        CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 14, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_RELEASE_PARALLEL_PORT_INFO           CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 15, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_GET_MORE_PARALLEL_PORT_INFO          CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 17, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_PARALLEL_SET_EPP_MODE                CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 18, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_PARALLEL_CLEAR_EPP_MODE              CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 19, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_GET_PARALLEL_PNP_INFO                CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 20, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_INTERNAL_PARCLASS_CONNECT                     CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 30, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_PARCLASS_DISCONNECT                  CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 31, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_DISCONNECT_IDLE                      CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 32, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_IEEE1284_NEGOTIATE                   CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 33, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_IEEE1284_GET_MODE                    CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 34, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_SET_READ_ADDRESS                     CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 35, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_INTERNAL_SET_WRITE_ADDRESS                    CTL_CODE(FILE_DEVICE_PARALLEL_PORT, 36, METHOD_BUFFERED, FILE_ANY_ACCESS)

typedef
BOOLEAN
(*PPARALLEL_TRY_ALLOCATE_ROUTINE) (
    IN  PVOID   TryAllocateContext
    );

typedef
VOID
(*PPARALLEL_FREE_ROUTINE) (
    IN  PVOID   FreeContext
    );

typedef
ULONG
(*PPARALLEL_QUERY_WAITERS_ROUTINE) (
    IN  PVOID   QueryAllocsContext
    );

//
// Hardware Capabilities
//
#define PPT_ECP_PRESENT             0x00000001
#define PPT_EPP_PRESENT             0x00000002

typedef struct _PARALLEL_PORT_INFORMATION {
    PHYSICAL_ADDRESS                OriginalController;
    PUCHAR                          Controller;
    ULONG                           SpanOfController;
    PPARALLEL_TRY_ALLOCATE_ROUTINE  TryAllocatePort;
    PPARALLEL_FREE_ROUTINE          FreePort;
    PPARALLEL_QUERY_WAITERS_ROUTINE QueryNumWaiters;
    PVOID                           Context;
} PARALLEL_PORT_INFORMATION, *PPARALLEL_PORT_INFORMATION;

typedef struct _PARALLEL_PNP_INFORMATION {
    ULONG                           PortNumber;
    ULONG                           HardwareCapabilities;
} PARALLEL_PNP_INFORMATION, *PPARALLEL_PNP_INFORMATION;

//
// The following structure is passed in on an
// IOCTL_INTERNAL_PARALLEL_CONNECT_INTERRUPT and on an
// IOCTL_INTERNAL_PARALLEL_DISCONNECT_INTERRUPT request.
//

typedef
VOID
(*PPARALLEL_DEFERRED_ROUTINE) (
    IN  PVOID   DeferredContext
    );

typedef struct _PARALLEL_INTERRUPT_SERVICE_ROUTINE {
    PKSERVICE_ROUTINE           InterruptServiceRoutine;
    PVOID                       InterruptServiceContext;
    PPARALLEL_DEFERRED_ROUTINE  DeferredPortCheckRoutine;   /* OPTIONAL */
    PVOID                       DeferredPortCheckContext;   /* OPTIONAL */
} PARALLEL_INTERRUPT_SERVICE_ROUTINE, *PPARALLEL_INTERRUPT_SERVICE_ROUTINE;

//
// The following structure is returned on an
// IOCTL_INTERNAL_PARALLEL_CONNECT_INTERRUPT request;
//

typedef struct _PARALLEL_INTERRUPT_INFORMATION {
    PKINTERRUPT                     InterruptObject;
    PPARALLEL_TRY_ALLOCATE_ROUTINE  TryAllocatePortAtInterruptLevel;
    PPARALLEL_FREE_ROUTINE          FreePortFromInterruptLevel;
    PVOID                           Context;
} PARALLEL_INTERRUPT_INFORMATION, *PPARALLEL_INTERRUPT_INFORMATION;

//
// The following structure is returned on an
// IOCTL_INTERNAL_GET_MORE_PARALLEL_PORT_INFO.
//

typedef struct _MORE_PARALLEL_PORT_INFORMATION {
    INTERFACE_TYPE  InterfaceType;
    ULONG           BusNumber;
    ULONG           InterruptLevel;
    ULONG           InterruptVector;
    KAFFINITY       InterruptAffinity;
    KINTERRUPT_MODE InterruptMode;
} MORE_PARALLEL_PORT_INFORMATION, *PMORE_PARALLEL_PORT_INFORMATION;

//
// The following structure is passed in to
// IOCTL_INTERNAL_PARCLASS_CONNECT.
//

typedef
NTSTATUS
(*PFILTER_CALLBACK) (
    IN     USHORT     Mode,
    IN     PVOID      CallbackContext,
    IN     PIRP       Irp,
    IN OUT BOOLEAN    *Connected
    );

typedef 
NTSTATUS
(*PNEGOTIATE_IEEE_MODE) (
    IN PVOID      Context,
    IN UCHAR      Extensibility
    );
    
typedef 
NTSTATUS
(*PTERMINATE_IEEE_MODE) (
    IN PVOID      Context
    );
    
typedef 
NTSTATUS
(*PTERMINATE_PARCLASS_MODE) (
    IN PVOID      Context
    );
    
//
// Callback Modes
//

#define MODE_PROCESS_IRP            0
#define MODE_SETUP_NEGOTIATE        1           /* PIRP Must be NULL. */
#define MODE_TERMINATE_FILTER_MODE  2           /* PIRP Must be NULL. */

typedef struct _PARCLASS_CONNECT_INFORMATION {
    PFILTER_CALLBACK    FilterCallback;
    PVOID               FilterContext;
} PARCLASS_CONNECT_INFORMATION, *PPARCLASS_CONNECT_INFORMATION;

typedef struct _PARCLASS_INFORMATION {
    PUCHAR                  Controller;
    ULONG                   SpanOfController;
    PNEGOTIATE_IEEE_MODE    NegotiateIeeeMode;
    PTERMINATE_IEEE_MODE    TerminateIeeeMode;
    PTERMINATE_IEEE_MODE    TerminateParclassMode;
    PVOID                   ParclassContext;
    ULONG                   HardwareCapabilities;
} PARCLASS_INFORMATION, *PPARCLASS_INFORMATION;

//
// These constants are used to construct the mask for the negotiation IOCTL.
// These need to be moved to ntddpar.h so that they are available to User Mode...
// IOCTL_INTERNAL_IEEE1284_NEGOTIATE.
//

typedef struct _PARCLASS_NEGOTIATION_MASK {
    USHORT      usReadMask;
    USHORT      usWriteMask;
} PARCLASS_NEGOTIATION_MASK, *PPARCLASS_NEGOTIATION_MASK;

#define NONE            0x0000
#define CENTRONICS      0x0001       /* Write Only */
#define NIBBLE          0x0002       /* Read Only */
#define BYTE            0x0004       /* Read Only */
#define EPP_HW          0x0008   
#define EPP_SW          0x0010
#define EPP_ANY         0x0018
#define ECP_HW          0x0020   
#define ECP_HW_NOIRQ    0x0040
#define ECP_SW          0x0080
#define ECP_ANY         0x00E0

//
// Standard and ECP parallel port offsets.
//

#define DATA_OFFSET         0
#define AFIFO_OFFSET        0
#define DSR_OFFSET          1
#define DCR_OFFSET          2
#define ECP_OFFSET          0x400
#define FIFO_OFFSET         0x400
#define CNFGA_OFFSET        0x400
#define CNFGB_OFFSET        0x401
#define ECR_OFFSET          0x402

#define ECP_SPAN            3

//
// Bit definitions for the DSR.
//

#define DSR_NOT_BUSY            0x80
#define DSR_NOT_ACK             0x40
#define DSR_PERROR              0x20
#define DSR_SELECT              0x10
#define DSR_NOT_FAULT           0x08

//
// More bit definitions for the DSR.
//

#define DSR_NOT_PTR_BUSY        0x80
#define DSR_NOT_PERIPH_ACK      0x80
#define DSR_WAIT                0x80
#define DSR_PTR_CLK             0x40
#define DSR_PERIPH_CLK          0x40
#define DSR_INTR                0x40
#define DSR_ACK_DATA_REQ        0x20
#define DSR_NOT_ACK_REVERSE     0x20
#define DSR_XFLAG               0x10
#define DSR_NOT_DATA_AVAIL      0x08
#define DSR_NOT_PERIPH_REQUEST  0x08

//
// Bit definitions for the DCR.
//

#define DCR_RESERVED            0xC0
#define DCR_DIRECTION           0x20
#define DCR_ACKINT_ENABLED      0x10
#define DCR_SELECT_IN           0x08
#define DCR_NOT_INIT            0x04
#define DCR_AUTOFEED            0x02
#define DCR_STROBE              0x01

//
// More bit definitions for the DCR.
//

#define DCR_NOT_1284_ACTIVE     0x08
#define DCR_ASTRB               0x08
#define DCR_NOT_REVERSE_REQUEST 0x04
#define DCR_NOT_HOST_BUSY       0x02
#define DCR_NOT_HOST_ACK        0x02
#define DCR_DSTRB               0x02
#define DCR_NOT_HOST_CLK        0x01
#define DCR_WRITE               0x01

//
// Bit definitions for configuration register A.
//

#define CNFGA_IMPID_MASK        0x70
#define CNFGA_IMPID_16BIT       0x00
#define CNFGA_IMPID_8BIT        0x10
#define CNFGA_IMPID_32BIT       0x20

#define CNFGA_NO_TRANS_BYTE     0x04

//
// Bit definitions for ECR register.
//

#define ECR_MODE_MASK           0xE0
#define ECR_SPP_MODE            0x00
#define ECR_BYTE_MODE           0x20
#define ECR_FASTCENT_MODE       0x40
#define ECR_ECP_MODE            0x60
#define ECR_EPP_MODE            0x80
#define ECR_TEST_MODE           0xC0

#define ECR_CONFIG_MODE         0xE0
#define ECR_ERRINT_DISABLED     0x10
#define ECR_DMA_ENABLED         0x08
#define ECR_SERVICE_INTERRUPT   0x04

#define ECR_FIFO_FULL           0x02
#define ECR_FIFO_EMPTY          0x01

//
// Full Mode definitions
//
#define ECP_PS2_MODE            0x34
#define ECP_COMPAT_MODE         0x14

#endif // _PARALLEL_
