
/*++

Copyright (c) 1997    Microsoft Corporation

Module Name:

    sample.h

Abstract:

    Header file for the sample DDK PnP driver modules. 

Environment:

    Kernel mode

Revision History:

    25-July-1997    :
        - Moved the declarations from sample.c
        - Added SYSTEM_POWER_STATE and DEVICE_POWER_STATE in the 
        device extension
         
    
    
--*/
    
        
#if     !defined(__SAMPLE_H__)
#define __SAMPLE_H__

#include "stdarg.h"
#include "stdio.h"
#include "stddef.h"

#include <wdm.h>

#ifdef  POOL_TAGGING
#ifdef  ExAllocatePool
#undef  ExAllocatePool
#endif
#define ExAllocatePool(a,b) ExAllocatePoolWithTag(a, b, 'pmaS')
#endif


#if DBG
#define SD_DebugPrint(_x_) \
               DbgPrint ("SD: "); \
               SampleDriverDbgPrint _x_;
VOID
SampleDriverDbgPrint    (
    ULONG   DebugPrintLevel,
    PCCHAR  DebugMessage,
    ...
    )   ;

#define TRAP() DbgBreakPoint()

#else
#define SD_DebugPrint(_x_)
#define TRAP()

#endif

//
// The device extension for the device object 
//
typedef struct _SD_FDO_DATA
{
    BOOLEAN             IsStarted; // This flag is set when is started.
                                   
    BOOLEAN             IsRemoved; // This flag is set when the device 
                                   // is removed.
    BOOLEAN             HoldNewRequests; // This flag is set whenever the
                                         // device needs to queue incoming
                                         // requests (when it receives a
                                         // QUERY_STOP or QUERY_REMOVE).
    
    LIST_ENTRY          NewRequestsQueue; // The queue where the incoming
                                          // requests are held when
                                          // HoldNewRequests is set.
    KSPIN_LOCK          QueueLock;        // The spin lock that protects
                                          // the queue
    
    PDEVICE_OBJECT      Self; // a back pointer to the DeviceObject.

    PDEVICE_OBJECT      PDO; // The PDO to which the FDO is attached.

    PDEVICE_OBJECT      NextLowerDriver; // The top of the device stack just
                                         // beneath this device object.

    KEVENT              RemoveEvent; // an event to sync outstandIO to zero.

    KEVENT              StopEvent;  // an event to sync outstandIO to 1.
    
    ULONG               OutstandingIO; // 1-biased count of reasons why
                                       // this object should stick around.
    
    SYSTEM_POWER_STATE  SystemPowerState;   // The general state
    DEVICE_POWER_STATE  DevicePowerState;   // The state of the device

}  SD_FDO_DATA, *PSD_FDO_DATA;


NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT  DriverObject,
    IN PUNICODE_STRING RegistryPath
    )   ;


NTSTATUS
SD_AddDevice(
    IN PDRIVER_OBJECT DriverObject,
    IN PDEVICE_OBJECT PhysicalDeviceObject
    )   ;


NTSTATUS
SD_Pass (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );


NTSTATUS
SD_DispatchPower (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );


NTSTATUS
SD_DispatchPnp (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );


NTSTATUS
SD_CreateClose (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );


NTSTATUS
SD_Cleanup (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );


NTSTATUS
SD_StartDevice (
    IN PSD_FDO_DATA     FdoData,
    IN PIRP             Irp
    );



NTSTATUS
SD_DispatchPnpComplete (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
    IN PVOID Context
    );


VOID
SD_Unload(
    IN PDRIVER_OBJECT DriverObject
    );



VOID
SD_CancelQueued (
    IN PDEVICE_OBJECT   DeviceObject,
    IN PIRP             Irp
    );



LONG
SD_IoIncrement    (
    IN  OUT PSD_FDO_DATA   FdoData
    )   ;


LONG
SD_IoDecrement    (
    IN  OUT PSD_FDO_DATA   FdoData
    )   ;


#endif  // __SAMPLE_H__
