/*++

Copyright (c) 1990, 1991, 1992, 1993  Microsoft Corporation

Module Name:

    i8042prt.h

Abstract:

    These are the structures and defines that are used in the
    Intel i8042 port driver.

Revision History:

--*/

#ifndef _I8042PRT_
#define _I8042PRT_

#include <ntddkbd.h>
#include <ntddmou.h>
#include "kbdmou.h"
#include "i8042cfg.h"
#include "i8042str.h"

#ifdef PNP_IDENTIFY
#include "devdesc.h"
#endif

//
// Define the timer values.
//

#define I8042_ASYNC_NO_TIMEOUT -1
#define I8042_ASYNC_TIMEOUT     3

//
// Define the default number of entries in the input data queue.
//

#define DATA_QUEUE_SIZE    100

//
// Define the default stall value.
//

#define I8042_STALL_DEFAULT      50

//
// Define the default "sync time" used to determine when the start
// of a new mouse data packet is expected.  The value is in units
// of 100 nanoseconds.
//

#define MOUSE_SYNCH_PACKET_100NS 10000000UL // 1 second, in 100 ns units

//
// Define booleans.
//

#define WAIT_FOR_ACKNOWLEDGE    TRUE
#define NO_WAIT_FOR_ACKNOWLEDGE FALSE
#define AND_OPERATION           TRUE
#define OR_OPERATION            FALSE
#define ENABLE_OPERATION        TRUE
#define DISABLE_OPERATION       FALSE

//
// Default keyboard scan code mode.
//

#define KEYBOARD_SCAN_CODE_SET 0x01

//
// Default number of function keys, number of LED indicators, and total
// number of keys located on the known types of keyboard.
//

#ifdef WINDOWS_FE
#define NUM_KNOWN_KEYBOARD_TYPES                   8
#else
#define NUM_KNOWN_KEYBOARD_TYPES                   4
#endif // WINDOWS_FE
#if defined(NEC_98)
#define KEYBOARD_TYPE_DEFAULT                      7
#define KEYBOARD_INDICATORS_DEFAULT                0x70
#else // defined(NEC_98)
#define KEYBOARD_TYPE_DEFAULT                      4
#define KEYBOARD_INDICATORS_DEFAULT                0
#endif // defined(NEC_98)

typedef struct _KEYBOARD_TYPE_INFORMATION {
    USHORT NumberOfFunctionKeys;
    USHORT NumberOfIndicators;
    USHORT NumberOfKeysTotal;
} KEYBOARD_TYPE_INFORMATION, *PKEYBOARD_TYPE_INFORMATION;

static const
KEYBOARD_TYPE_INFORMATION KeyboardTypeInformation[NUM_KNOWN_KEYBOARD_TYPES] = {
    {10, 3, 84},     // PC/XT 83- 84-key keyboard (and compatibles)
    {12, 3, 102},    // Olivetti M24 102-key keyboard (and compatibles)
    {10, 3, 84},     // All AT type keyboards (84-86 keys)
    {12, 3, 101}     // Enhanced 101- or 102-key keyboards (and compatibles)
#ifdef WINDOWS_FE
   ,{12, 3, 101},    // 5:
    {12, 3, 101},    // 6:
#if defined(NEC_98)
    {15, 2, 106},    // 7:  NEC 106 keyboard
#else // defined(NEC_98)
    { 0, 0, 0},      // 7: Japanese Keyboard
#endif // defined(NEC_98)
    { 0, 0, 0}       // 8: Korean keyboard
#endif // WINDOWS_FE
};

#ifdef WINDOWS_FE

#define IBM02_KEYBOARD(Id) ((Id).Subtype == 1)
#define AX_KEYBOARD(Id)    ((Id).Subtype == 3)

#define FE_SUBTYPE(SubType,OemId) ((SubType)|((OemId<<4)))

typedef struct _KEYBOARD_OEM_INFORMATION {
    KEYBOARD_ID               KeyboardId;
    KEYBOARD_TYPE_INFORMATION KeyboardTypeInformation;
} KEYBOARD_OEM_INFORMATION, *PKEYBOARD_OEM_INFORMATION;

static const
KEYBOARD_OEM_INFORMATION KeyboardFarEastOemInformation[] = {
#if defined(NEC_98)
    {{7, FE_SUBTYPE(1,0)}, {15,2,106}}, // PC-9800 Standard keyboard
    {{7, FE_SUBTYPE(2,0)}, {10,2,106}}, // PC-9801 VX/UX,PC-98XL/XL2(Normal mode) keyboard
    {{7, FE_SUBTYPE(3,0)}, {15,2,106}}, // PC-98XL/XL2(Hireso mode) keyboard
    {{7, FE_SUBTYPE(4,0)}, {15,2,106}}, // PC-9800 Laptop keyboard
    {{7, FE_SUBTYPE(5,0)}, {12,2,106}}, // PC-9800 N106(PAO) keyboard
    {{0, FE_SUBTYPE(0,0)}, { 0,0,  0}}  // Array terminator
#else // defined(NEC_98)
    {{7, FE_SUBTYPE(1,0)}, {12,3,101}}, // PC/AT 101 Enhanced Japanese Keyboard
    {{7, FE_SUBTYPE(1,0)}, {12,4,105}}, // AX standard Japanese keyboard
    {{7, FE_SUBTYPE(2,0)}, {12,3,106}}, // PC/AT 106 Japanese Keyboard
    {{7, FE_SUBTYPE(3,0)}, {12,3,106}}, // IBM 5576-002 keyboard
    {{7, FE_SUBTYPE(1,1)}, {12,4,105}}, // AX consortium compatible keyboard
    {{8, FE_SUBTYPE(3,0)}, {12,3,101}}, // PC/AT 101 Enhanced Korean Keyboard (A)
    {{8, FE_SUBTYPE(4,0)}, {12,3,101}}, // PC/AT 101 Enhanced Korean Keyboard (B)
    {{8, FE_SUBTYPE(5,0)}, {12,3,101}}, // PC/AT 101 Enhanced Korean Keyboard (C)
    {{8, FE_SUBTYPE(6,0)}, {12,3,103}}, // PC/AT 103 Enhanced Korean Keyboard
    {{0, FE_SUBTYPE(0,0)}, { 0,0,  0}}  // Array terminator
#endif // defined(NEC_98)
};

#endif // WINDOWS_FE

//
// Minimum, maximum, and default values for keyboard typematic rate and delay.
//

#define KEYBOARD_TYPEMATIC_RATE_MINIMUM     2
#define KEYBOARD_TYPEMATIC_RATE_MAXIMUM    30
#define KEYBOARD_TYPEMATIC_RATE_DEFAULT    30
#define KEYBOARD_TYPEMATIC_DELAY_MINIMUM  250
#define KEYBOARD_TYPEMATIC_DELAY_MAXIMUM 1000
#define KEYBOARD_TYPEMATIC_DELAY_DEFAULT  250

//
// Define the 8042 mouse status bits.
//

#define LEFT_BUTTON_DOWN   0x01
#define RIGHT_BUTTON_DOWN  0x02
#define MIDDLE_BUTTON_DOWN 0x04
#define X_DATA_SIGN        0x10
#define Y_DATA_SIGN        0x20
#define X_OVERFLOW         0x40
#define Y_OVERFLOW         0x80

#define MOUSE_SIGN_OVERFLOW_MASK (X_DATA_SIGN | Y_DATA_SIGN | X_OVERFLOW | Y_OVERFLOW)

//
// Define the maximum positive and negative values for mouse motion.
//

#define MOUSE_MAXIMUM_POSITIVE_DELTA 0x000000FF
#define MOUSE_MAXIMUM_NEGATIVE_DELTA 0xFFFFFF00

//
// Default number of buttons and sample rate for the i8042 mouse.
//

#define MOUSE_NUMBER_OF_BUTTONS     2
#define MOUSE_SAMPLE_RATE           60

//
// Define the mouse resolution specifier.  Note that (2**MOUSE_RESOLUTION)
// specifies counts-per-millimeter.  Counts-per-centimeter is
// (counts-per-millimeter * 10).
//

#define MOUSE_RESOLUTION            3

//
// Define the maximum number of resets we allow without success before we
// give up and consider the mouse dead
//
#define MOUSE_NUMBER_OF_RESETS      5

//
// Defines for DeviceExtension->HardwarePresent.
//

#define KEYBOARD_HARDWARE_PRESENT   0x01
#define MOUSE_HARDWARE_PRESENT      0x02
#define BALLPOINT_HARDWARE_PRESENT  0x04
#define WHEELMOUSE_HARDWARE_PRESENT 0x08

//
// Define macros for performing I/O on the 8042 command/status and data
// registers.
//

#define I8X_PUT_COMMAND_BYTE(Address, Byte)                              \
    WRITE_PORT_UCHAR(Address, (UCHAR) Byte)
#define I8X_PUT_DATA_BYTE(Address, Byte)                                 \
    WRITE_PORT_UCHAR(Address, (UCHAR) Byte)
#define I8X_GET_STATUS_BYTE(Address)                                     \
    READ_PORT_UCHAR(Address)
#define I8X_GET_DATA_BYTE(Address)                                       \
    READ_PORT_UCHAR(Address)
#define I8X_WRITE_CMD_TO_MOUSE( Extension )                              \
    I8xPutByteAsynchronous(                                              \
        (CCHAR) CommandPort,                                             \
        Extension,                                                       \
        (UCHAR) I8042_WRITE_TO_AUXILIARY_DEVICE                          \
        );
#define I8X_MOUSE_COMMAND( Extension, Byte )                             \
    I8xPutByteAsynchronous(                                              \
        (CCHAR) DataPort,                                                \
        Extension,                                                       \
        (UCHAR) Byte                                                     \
        );

//
// Define commands to the 8042 controller.
//

#define I8042_READ_CONTROLLER_COMMAND_BYTE      0x20
#define I8042_WRITE_CONTROLLER_COMMAND_BYTE     0x60
#define I8042_DISABLE_MOUSE_DEVICE              0xA7
#define I8042_ENABLE_MOUSE_DEVICE               0xA8
#define I8042_AUXILIARY_DEVICE_TEST             0xA9
#define I8042_KEYBOARD_DEVICE_TEST              0xAB
#define I8042_DISABLE_KEYBOARD_DEVICE           0xAD
#define I8042_ENABLE_KEYBOARD_DEVICE            0xAE
#define I8042_WRITE_TO_AUXILIARY_DEVICE         0xD4

//
// Define the 8042 Controller Command Byte.
//

#define CCB_ENABLE_KEYBOARD_INTERRUPT 0x01
#define CCB_ENABLE_MOUSE_INTERRUPT    0x02
#define CCB_DISABLE_KEYBOARD_DEVICE   0x10
#define CCB_DISABLE_MOUSE_DEVICE      0x20
#define CCB_KEYBOARD_TRANSLATE_MODE   0x40


//
// Define the 8042 Controller Status Register bits.
//

#define OUTPUT_BUFFER_FULL       0x01
#define INPUT_BUFFER_FULL        0x02
#define MOUSE_OUTPUT_BUFFER_FULL 0x20

//
// Define the 8042 responses.
//
#if defined(NEC_98)
#define ACKNOWLEDGE               0xFA
#define RESEND                    0xFC

//
// Define commands to the keyboard (through the 8042 data port).
//
#define SET_KEYBOARD_INDICATORS           0x9D
#define GET_KEYBOARD_INDICATORS           0x60
#define SELECT_SCAN_CODE_SET              0xF0
#define READ_KEYBOARD_ID                  0x9F
#define READ_KEYBOARD_2ND_ID              0x96
#define SET_KEYBOARD_TYPEMATIC            0x9C
#define SET_ALL_TYPEMATIC_MAKE_BREAK      0xFA
#define KEYBOARD_RESET                    0xFF

#define PC98_STOP_AUTO_REPEAT             0x70
#define SET_PAO_KEYBOARD_MODE             0x94
#define SET_PAO_KEYBOARD_MODE_VMODE       0x72
#define SET_WIN95_KEYBOARD_MODE           0x95
#define SET_WIN95_KEYBOARD_MODE_NOMAL     0x01 // Win95 Normal 98
#define SET_WIN95_KEYBOARD_MODE_SHIFT     0x02 // Win95 Shift separate
#define SET_WIN95_KEYBOARD_MODE_WIN95     0x03 // Win95 Win95 mode

//
// keyboard 2nd ID request.
//
#define PC98_KEYBOARD_ID_FIRST_BYTE       0xA0
#define PC98_KEYBOARD_ID_2ND_BYTE_STD     0x80
#define PC98_KEYBOARD_ID_2ND_BYTE_PTOS    0x81
#define PC98_KEYBOARD_ID_2ND_BYTE_N106    0x82
#define PC98_KEYBOARD_ID_2ND_BYTE_Win95   0x83
#else // defined(NEC_98)
#define ACKNOWLEDGE         0xFA
#define RESEND              0xFE

//
// Define commands to the keyboard (through the 8042 data port).
//

#define SET_KEYBOARD_INDICATORS           0xED
#define SELECT_SCAN_CODE_SET              0xF0
#define READ_KEYBOARD_ID                  0xF2
#define SET_KEYBOARD_TYPEMATIC            0xF3
#define SET_ALL_TYPEMATIC_MAKE_BREAK      0xFA
#define KEYBOARD_RESET                    0xFF

#endif // defined(NEC_98)
//
// Define the keyboard responses.
//

#define KEYBOARD_COMPLETE_SUCCESS 0xAA
#define KEYBOARD_COMPLETE_FAILURE 0xFC
#define KEYBOARD_BREAK_CODE       0xF0
#define KEYBOARD_DEBUG_HOTKEY_ENH 0x37 // SysReq scan code for Enhanced Keyboard
#define KEYBOARD_DEBUG_HOTKEY_AT  0x54 // SysReq scan code for 84-key Keyboard

//
// Define commands to the mouse (through the 8042 data port).
//

#define SET_MOUSE_RESOLUTION              0xE8
#define SET_MOUSE_SAMPLING_RATE           0xF3
#define MOUSE_RESET                       0xFF
#define ENABLE_MOUSE_TRANSMISSION         0xF4
#define SET_MOUSE_SCALING_1TO1            0xE6
#define READ_MOUSE_STATUS                 0xE9
#define GET_DEVICE_ID                     0xF2

//
// Define the mouse responses.
//

#define MOUSE_COMPLETE      0xAA
#define MOUSE_ID_BYTE       0x00
#define WHEELMOUSE_ID_BYTE  0x03

//
// Define the i8042 controller input/output ports.
//

typedef enum _I8042_IO_PORT_TYPE {
    DataPort = 0,
    CommandPort,
    MaximumPortCount

} I8042_IO_PORT_TYPE;

//
// Define the device types attached to the i8042 controller.
//

typedef enum _I8042_DEVICE_TYPE {
    ControllerDeviceType,
    KeyboardDeviceType,
    MouseDeviceType,
    UndefinedDeviceType
} I8042_DEVICE_TYPE;

//
// Define the keyboard output states.
//

typedef enum _KEYBOARD_STATE {
    Idle,
    SendFirstByte,
    SendLastByte
} KEYBOARD_STATE;

//
// Define the keyboard scan code input states.
//

typedef enum _KEYBOARD_SCAN_STATE {
    Normal,
    GotE0,
    GotE1
} KEYBOARD_SCAN_STATE;

//
// Define the mouse states.
//

typedef enum _MOUSE_STATE {
    MouseIdle,
    XMovement,
    YMovement,
    ZMovement,
    MouseExpectingACK,
    MouseResetting
} MOUSE_STATE;

#if defined(NEC_98)
//
// procedures to translate scancode(PC9800 -> PC/AT 106)
//
typedef NTSTATUS (*LPSCANCODEPROC)(UCHAR*, USHORT*);
#endif // defined(NEC_98)
//
// When we detect a (possible) reset condition from the mouse
// input stream, we enter the following substates
//
typedef enum _MOUSE_RESET_SUBSTATE {
    ExpectingReset,
    ExpectingResetId,
    ExpectingGetDeviceIdACK,
    ExpectingGetDeviceId,
    ExpectingEnableACK,
    ExpectingSetSamplingRate20HzACK,
    ExpectingSetSamplingRate20Hz,
    ExpectingSetSamplingRate40HzACK,
    ExpectingSetSamplingRate40Hz,
    ExpectingSetSamplingRate60HzACK,
    ExpectingSetSamplingRate60Hz,
    ExpectingPnpIdByte1,
    ExpectingPnpIdByte2,
    ExpectingPnpIdByte3,
    ExpectingPnpIdByte4,
    ExpectingPnpIdByte5,
    ExpectingPnpIdByte6,
    ExpectingPnpIdByte7,
    ExpectingSetSamplingRate200HzACK,
    ExpectingSetSamplingRate200Hz,
    ExpectingSetSamplingRate100HzACK,
    ExpectingSetSamplingRate100Hz,
    ExpectingSetSamplingRate80HzACK,
    ExpectingSetSamplingRate80Hz,
    ExpectingGetDeviceId2ACK,
    ExpectingGetDeviceId2,
    ExpectingSetSamplingRateDefaultACK,
    ExpectingSetSamplingRateDefault
} MOUSE_RESET_SUBSTATE;

//
// Define the keyboard set request packet.
//

typedef struct _KEYBOARD_SET_PACKET {
    USHORT State;
    UCHAR  FirstByte;
    UCHAR  LastByte;
} KEYBOARD_SET_PACKET, *PKEYBOARD_SET_PACKET;

//
// Intel i8042 configuration information.
//

typedef struct _I8042_CONFIGURATION_INFORMATION {

    //
    // Bus interface type.
    //
    INTERFACE_TYPE InterfaceType;

    //
    // Bus Number.
    //
    ULONG BusNumber;

    //
    // The port/register resources used by this device.
    //
    CM_PARTIAL_RESOURCE_DESCRIPTOR PortList[MaximumPortCount];
    ULONG PortListCount;

    //
    // Keyboard interrupt resources.
    //
    CM_PARTIAL_RESOURCE_DESCRIPTOR KeyboardInterrupt;

    //
    // Mouse interrupt resources.
    //
    CM_PARTIAL_RESOURCE_DESCRIPTOR MouseInterrupt;

    //
    // Flag that indicates whether floating point context should be saved.
    //
    BOOLEAN FloatingSave;

    //
    // Number of retries allowed.
    //
    USHORT ResendIterations;

    //
    // Number of polling iterations allowed.
    //
    USHORT PollingIterations;

    //
    // Maximum number of polling iterations allowed.
    //
    USHORT PollingIterationsMaximum;

    //
    // Maximum number of times to check the Status register in
    // the ISR before deciding the interrupt is spurious.
    //
    USHORT PollStatusIterations;

    //
    // Microseconds to stall in KeStallExecutionProcessor calls.
    //
    USHORT StallMicroseconds;

    //
    // Keyboard attributes.
    //
    KEYBOARD_ATTRIBUTES KeyboardAttributes;

    //
    // Initial values of keyboard typematic rate and delay.
    //
    KEYBOARD_TYPEMATIC_PARAMETERS KeyRepeatCurrent;

    //
    // Current indicator (LED) setting.
    //
    KEYBOARD_INDICATOR_PARAMETERS KeyboardIndicators;

    //
    // Mouse attributes.
    //
    MOUSE_ATTRIBUTES MouseAttributes;
    USHORT MouseResolution;

    //
    // Boolean flags determines whether we should try and detect the wheel
    // on the mouse or not
    //
    ULONG EnableWheelDetection;

} I8042_CONFIGURATION_INFORMATION, *PI8042_CONFIGURATION_INFORMATION;

//
// Define the keyboard portion of the port device extension.
//

typedef struct _PORT_KEYBOARD_EXTENSION {

    //
    // Keyboard class connection data.
    //
    CONNECT_DATA ConnectData;

    //
    // Number of input data items currently in the keyboard InputData queue.
    //
    ULONG InputCount;

    //
    // Start of the port keyboard input data queue (really a circular buffer).
    //
    PKEYBOARD_INPUT_DATA InputData;

    //
    // Insertion pointer for keyboard InputData.
    //
    PKEYBOARD_INPUT_DATA DataIn;

    //
    // Removal pointer for keyboard InputData.
    //
    PKEYBOARD_INPUT_DATA DataOut;

    //
    // Points one input packet past the end of the InputData buffer.
    //
    PKEYBOARD_INPUT_DATA DataEnd;

    //
    // Current keyboard input packet.
    //
    KEYBOARD_INPUT_DATA CurrentInput;

    //
    // Current keyboard scan input state.
    //
    KEYBOARD_SCAN_STATE CurrentScanState;

    //
    // Current keyboard output packet (for set requests).
    //
    KEYBOARD_SET_PACKET CurrentOutput;

    //
    // Current resend count.
    //
    USHORT ResendCount;

    //
    // Request sequence number (used for error logging).
    //
    ULONG SequenceNumber;

    //
    // Timer used to retry the ISR DPC routine when the class
    // driver is unable to consume all the port driver's data.
    //
    KTIMER DataConsumptionTimer;

    //
    // Indicates which keyboard port device this driver created (UnitId
    // is the suffix appended to the keyboard port basename for the
    // call to IoCreateDevice).
    //
    USHORT UnitId;

    //
    // Indicates whether it is okay to log overflow errors.
    //
    BOOLEAN OkayToLogOverflow;

} PORT_KEYBOARD_EXTENSION, *PPORT_KEYBOARD_EXTENSION;

//
// Define the mouse portion of the port device extension.
//

typedef struct _PORT_MOUSE_EXTENSION {

    //
    // Mouse class connection data.
    //
    CONNECT_DATA ConnectData;

    //
    // Number of input data items currently in the mouse InputData queue.
    //
    ULONG InputCount;

    //
    // Start of the port mouse input data queue (really a circular buffer).
    //
    PMOUSE_INPUT_DATA InputData;

    //
    // Insertion pointer for mouse InputData.
    //
    PMOUSE_INPUT_DATA DataIn;

    //
    // Removal pointer for mouse InputData.
    //
    PMOUSE_INPUT_DATA DataOut;

    //
    // Points one input packet past the end of the InputData buffer.
    //
    PMOUSE_INPUT_DATA DataEnd;

    //
    // Current mouse input packet.
    //
    MOUSE_INPUT_DATA CurrentInput;

    //
    // Current mouse input state.
    //
    USHORT InputState;
    USHORT InputResetSubState;

    //
    // Count the number of times we have reset and failed
    //
    USHORT ResetCount;

    //
    // Current mouse sign and overflow data.
    //
    UCHAR CurrentSignAndOverflow;

    //
    // Previous mouse sign and overflow data.
    //
    UCHAR PreviousSignAndOverflow;

    //
    // Previous mouse button data.
    //
    UCHAR PreviousButtons;

    //
    // Request sequence number (used for error logging).
    //
    ULONG SequenceNumber;

    //
    // Timer used to retry the ISR DPC routine when the class
    // driver is unable to consume all the port driver's data.
    //
    KTIMER DataConsumptionTimer;

    //
    // The tick count (since system boot) at which the mouse last interrupted.
    // Retrieved via KeQueryTickCount.  Used to determine whether a byte of
    // the mouse data packet has been lost.  Allows the driver to synch
    // up with the true mouse input state.
    //
    LARGE_INTEGER PreviousTick;

    //
    // Indicates which pointer port device this driver created (UnitId
    // is the suffix appended to the pointer port basename for the
    // call to IoCreateDevice).
    //
    USHORT UnitId;

    //
    // Indicates whether it is okay to log overflow errors.
    //
    BOOLEAN OkayToLogOverflow;

    //
    // Number of interval timer ticks to wait before deciding that the
    // next mouse interrupt is for the start of a new packet.  Used to
    // synch up again if a byte of the mouse packet gets lost.
    //
    ULONG SynchTickCount;

    //
    // Keep track of last byte of data received from mouse so we can detect
    // the two-byte string which indicates a potential reset
    //
    UCHAR LastByteReceived;

} PORT_MOUSE_EXTENSION, *PPORT_MOUSE_EXTENSION;

//
// Port device extension.
//

typedef struct _DEVICE_EXTENSION {

    //
    // Indicate which hardware is actually present (keyboard and/or mouse).
    //
    ULONG HardwarePresent;

    //
    // Reference count for number of keyboard enables.
    //
    LONG KeyboardEnableCount;

    //
    // Reference count for number of mouse enables.
    //
    LONG MouseEnableCount;

    //
    // Pointer to the device object.
    //
    PDEVICE_OBJECT DeviceObject;

    //
    // The mapped addresses for this device's registers.
    //
    PUCHAR DeviceRegisters[MaximumPortCount];

    //
    // Keyboard-specific port connection data.
    //
    PORT_KEYBOARD_EXTENSION KeyboardExtension;

    //
    // Mouse-specific port connection data.
    //
    PORT_MOUSE_EXTENSION MouseExtension;

    //
    // Port configuration information.
    //
    I8042_CONFIGURATION_INFORMATION Configuration;

    //
    // i8042 keyboard and mouse interrupt objects.
    //
    PKINTERRUPT KeyboardInterruptObject;
    PKINTERRUPT MouseInterruptObject;
    KSPIN_LOCK SharedInterruptSpinLock;

    //
    // DPC queue for completion of requests that fail by exceeding
    // the maximum number of retries.
    //
    KDPC RetriesExceededDpc;

    //
    // DPC queue for logging overrun and internal driver errors.
    //
    KDPC ErrorLogDpc;
    KDPC ErrorLogMouseDpc;

    //
    // Keyboard ISR DPC queue.
    //
    KDPC KeyboardIsrDpc;

    //
    // Keyboard ISR DPC recall queue.
    //
    KDPC KeyboardIsrDpcRetry;

    //
    // Used by the ISR and the ISR DPC (in I8xDpcVariableOperation calls)
    // to control processing by the ISR DPC.
    //
    LONG DpcInterlockKeyboard;

    //
    // Mouse ISR DPC queue.
    //
    KDPC MouseIsrDpc;

    //
    // Mouse ISR DPC recall queue.
    //
    KDPC MouseIsrDpcRetry;

    //
    // Used by the ISR and the ISR DPC (in I8xDpcVariableOperation calls)
    // to control processing by the ISR DPC.
    //
    LONG DpcInterlockMouse;

    //
    // DPC queue for command timeouts.
    //
    KDPC TimeOutDpc;

    //
    // Timer used to timeout i8042 commands.
    //
    KTIMER CommandTimer;

    //
    // Timer count used by the command time out routine.
    //
    LONG TimerCount;

    //
    // Set at intialization to indicate that the register
    // addresses must be unmapped when the driver is unloaded.
    //
    BOOLEAN UnmapRegistersRequired;

#if defined(WINDOWS_FE) && defined(_X86_)
    //
    // Crash by key combination.
    //
    LONG Dump1Keys;           // CrashDump call first press keys flag
                              //  7 6 5 4 3 2 1 0 bit
                              //    | | |   | | +--- Right Shift Key
                              //    | | |   | +----- Right Ctrl Key
                              //    | | |   +------- Right Alt Key
                              //    | | +----------- Left Shift Key
                              //    | +------------- Left Ctrl Key
                              //    +--------------- Left Alt Key
    LONG Dump2Key;            // CrashDump call second twice press key no
    LONG DumpFlags;           // Key press flags
#endif // defined(WINDOWS_FE) && defined(_X86_)

#if defined(NEC_98)
    //
    // Keyboard type identifier.
    //
    ULONG KeyboardTypeNEC;

    //
    // A pointer to scancode translation procedure.
    //
    LPSCANCODEPROC ScancodeProc;

    //
    // Ctrl Key Status
    //
    USHORT CtrlKeyStatus;  // Bit 1 = Right Ctrl, Bit 5 = Left Ctrl

    //
    // VFKey Emulation Flag
    //
    // if 0(emulation off),  vf3 -> vf3, vf4 -> vf4, vf5 -> vf4
    // otherwise(emulation on),  vf3 -> ScrollLock, vf4 -> Pause, vf5 -> NumLock
    //
    ULONG VfKeyEmulation;

#define IsVfKeyEmulation   (deviceExtension->VfKeyEmulation != 0)

#endif // defined(NEC_98)
} DEVICE_EXTENSION, *PDEVICE_EXTENSION;

//
// Temporary structure used during initialization
//

typedef struct _INIT_EXTENSION {

    DEVICE_EXTENSION DeviceExtension;

#ifdef PNP_IDENTIFY
    HWDESC_INFO MouseConfig;
    HWDESC_INFO KeyboardConfig;
#endif

} INIT_EXTENSION, *PINIT_EXTENSION;

//
// Define the port InitializeDataQueue context structure.
//
typedef struct _I8042_INITIALIZE_DATA_CONTEXT {
    IN PDEVICE_EXTENSION DeviceExtension;
    IN CCHAR DeviceType;
} I8042_INITIALIZE_DATA_CONTEXT, *PI8042_INITIALIZE_DATA_CONTEXT;

//
// Define the port TransmitControllerCommandByte context structure.
//
typedef struct _I8042_TRANSMIT_CCB_CONTEXT {
    IN ULONG HardwareDisableEnableMask;
    IN BOOLEAN AndOperation;
    IN UCHAR ByteMask;
    OUT NTSTATUS Status;
} I8042_TRANSMIT_CCB_CONTEXT, *PI8042_TRANSMIT_CCB_CONTEXT;

//
// Define the port DeviceEnableDisable context structure.
//
typedef struct _I8042_DEVICE_ENABLE_DISABLE_CONTEXT {
    IN PDEVICE_EXTENSION DeviceExtension;
    IN ULONG EnableMask;
    IN BOOLEAN AndOperation;
    OUT NTSTATUS Status;
} I8042_DEVICE_ENABLE_DISABLE_CONTEXT,
  *PI8042_DEVICE_ENABLE_DISABLE_CONTEXT;

//
// Define the port Get/SetDataQueuePointer context structures.
//
typedef struct _GET_DATA_POINTER_CONTEXT {
    IN PDEVICE_EXTENSION DeviceExtension;
    IN CCHAR DeviceType;
    OUT PVOID DataIn;
    OUT PVOID DataOut;
    OUT ULONG InputCount;
} GET_DATA_POINTER_CONTEXT, *PGET_DATA_POINTER_CONTEXT;

typedef struct _SET_DATA_POINTER_CONTEXT {
    IN PDEVICE_EXTENSION DeviceExtension;
    IN CCHAR DeviceType;
    IN ULONG InputCount;
    IN PVOID DataOut;
} SET_DATA_POINTER_CONTEXT, *PSET_DATA_POINTER_CONTEXT;

//
// Define the port timer context structure.
//
typedef struct _TIMER_CONTEXT {
    IN PDEVICE_OBJECT DeviceObject;
    IN PLONG TimerCounter;
    OUT LONG NewTimerCount;
} TIMER_CONTEXT, *PTIMER_CONTEXT;

//
// Define the port KeyboardInitiate context structure.
//
typedef struct _KEYBOARD_INITIATE_CONTEXT {
    IN PDEVICE_OBJECT DeviceObject;
    IN UCHAR FirstByte;
    IN UCHAR LastByte;
} KEYBOARD_INITIATE_CONTEXT, *PKEYBOARD_INITIATE_CONTEXT;

//
// Statically allocate the (known) scancode-to-indicator-light mapping.
// This information is returned by the
// IOCTL_KEYBOARD_QUERY_INDICATOR_TRANSLATION device control request.
//

#define KEYBOARD_NUMBER_OF_INDICATORS              3

static const INDICATOR_LIST IndicatorList[KEYBOARD_NUMBER_OF_INDICATORS] = {
        {0x3A, KEYBOARD_CAPS_LOCK_ON},
        {0x45, KEYBOARD_NUM_LOCK_ON},
        {0x46, KEYBOARD_SCROLL_LOCK_ON}};

//
// Define the context structure and operations for I8xDpcVariableOperation.
//
typedef enum _OPERATION_TYPE {
        IncrementOperation,
        DecrementOperation,
        WriteOperation,
        ReadOperation
} OPERATION_TYPE;

typedef struct _VARIABLE_OPERATION_CONTEXT {
    IN PLONG VariableAddress;
    IN OPERATION_TYPE Operation;
    IN OUT PLONG NewValue;
} VARIABLE_OPERATION_CONTEXT, *PVARIABLE_OPERATION_CONTEXT;


//
// Function prototypes.
//
NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT DriverObject,
    IN PUNICODE_STRING RegistryPath
    );

VOID
I8042CompletionDpc(
    IN PKDPC Dpc,
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
    IN PVOID Context
    );

VOID
I8042ErrorLogDpc(
    IN PKDPC Dpc,
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
    IN PVOID Context
    );

NTSTATUS
I8042Flush(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

NTSTATUS
I8042InternalDeviceControl(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

BOOLEAN
I8042KeyboardInterruptService(
    IN PKINTERRUPT Interrupt,
    IN PVOID Context
    );

VOID
I8042KeyboardIsrDpc(
    IN PKDPC Dpc,
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
    IN PVOID Context
    );

BOOLEAN
I8042MouseInterruptService(
    IN PKINTERRUPT Interrupt,
    IN PVOID Context
    );

VOID
I8042MouseIsrDpc(
    IN PKDPC Dpc,
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
    IN PVOID Context
    );

NTSTATUS
I8042OpenCloseDispatch(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

VOID
I8042RetriesExceededDpc(
    IN PKDPC Dpc,
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
    IN PVOID Context
    );

VOID
I8042StartIo(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

VOID
I8042TimeOutDpc(
    IN PKDPC Dpc,
    IN PDEVICE_OBJECT DeviceObject,
    IN PVOID SystemContext1,
    IN PVOID SystemContext2
    );

VOID
I8042Unload(
    IN PDRIVER_OBJECT DriverObject
    );

VOID
I8xBuildResourceList(
    IN PDEVICE_EXTENSION DeviceExtension,
    OUT PCM_RESOURCE_LIST *ResourceList,
    OUT PULONG ResourceListSize
    );

UCHAR
I8xConvertTypematicParameters(
    IN USHORT Rate,
    IN USHORT Delay
    );

#if DBG

VOID
I8xDebugPrint(
    ULONG DebugPrintLevel,
    PCCHAR DebugMessage,
    ...
    );

extern ULONG i8042Debug;
#define I8xPrint(x) I8xDebugPrint x
#else
#define I8xPrint(x)
#endif

VOID
I8xDecrementTimer(
    IN PTIMER_CONTEXT Context
    );

VOID
I8xDeviceEnableDisable(
    IN PVOID Context
    );

VOID
I8xDpcVariableOperation(
    IN  PVOID Context
    );

VOID
I8xDrainOutputBuffer(
    IN PUCHAR DataAddress,
    IN PUCHAR CommandAddress
    );

VOID
I8xGetByteAsynchronous(
    IN CCHAR DeviceType,
    IN PDEVICE_EXTENSION DeviceExtension,
    OUT PUCHAR Byte
    );

NTSTATUS
I8xGetBytePolled(
    IN CCHAR DeviceType,
    IN PDEVICE_EXTENSION DeviceExtension,
    OUT PUCHAR Byte
    );

NTSTATUS
I8xGetControllerCommand(
    IN ULONG HardwareDisableEnableMask,
    IN PDEVICE_EXTENSION DeviceExtension,
    OUT PUCHAR Byte
    );

VOID
I8xGetDataQueuePointer(
    IN PVOID Context
    );

VOID
I8xInitializeDataQueue(
    IN PVOID Context
    );

VOID
I8xInitializeHardware(
    IN PDEVICE_OBJECT DeviceObject
    );

NTSTATUS
I8xInitializeKeyboard(
    IN PDEVICE_OBJECT DeviceObject
    );

NTSTATUS
I8xInitializeMouse(
    IN PDEVICE_OBJECT DeviceObject
    );

VOID
I8xKeyboardConfiguration(
    IN PINIT_EXTENSION InitializationData,
    IN PUNICODE_STRING RegistryPath,
    IN PUNICODE_STRING KeyboardDeviceName,
    IN PUNICODE_STRING PointerDeviceName
    );

VOID
I8xKeyboardInitiateIo(
    IN PVOID Context
    );

VOID
I8xKeyboardInitiateWrapper(
    IN PVOID Context
    );

NTSTATUS
I8xKeyboardPeripheralCallout(
    IN PVOID Context,
    IN PUNICODE_STRING PathName,
    IN INTERFACE_TYPE BusType,
    IN ULONG BusNumber,
    IN PKEY_VALUE_FULL_INFORMATION *BusInformation,
    IN CONFIGURATION_TYPE ControllerType,
    IN ULONG ControllerNumber,
    IN PKEY_VALUE_FULL_INFORMATION *ControllerInformation,
    IN CONFIGURATION_TYPE PeripheralType,
    IN ULONG PeripheralNumber,
    IN PKEY_VALUE_FULL_INFORMATION *PeripheralInformation
    );

VOID
I8xLogError(
    IN PDEVICE_OBJECT DeviceObject,
    IN NTSTATUS ErrorCode,
    IN ULONG UniqueErrorValue,
    IN NTSTATUS FinalStatus,
    IN PULONG DumpData,
    IN ULONG DumpCount
    );

VOID
I8xMouseConfiguration(
    IN PINIT_EXTENSION InitializationData,
    IN PUNICODE_STRING RegistryPath,
    IN PUNICODE_STRING KeyboardDeviceName,
    IN PUNICODE_STRING PointerDeviceName
    );

NTSTATUS
I8xMouseEnableTransmission(
    IN PDEVICE_OBJECT DeviceObject
    );

NTSTATUS
I8xMousePeripheralCallout(
    IN PVOID Context,
    IN PUNICODE_STRING PathName,
    IN INTERFACE_TYPE BusType,
    IN ULONG BusNumber,
    IN PKEY_VALUE_FULL_INFORMATION *BusInformation,
    IN CONFIGURATION_TYPE ControllerType,
    IN ULONG ControllerNumber,
    IN PKEY_VALUE_FULL_INFORMATION *ControllerInformation,
    IN CONFIGURATION_TYPE PeripheralType,
    IN ULONG PeripheralNumber,
    IN PKEY_VALUE_FULL_INFORMATION *PeripheralInformation
    );

VOID
I8xPutByteAsynchronous(
    IN CCHAR PortType,
    IN PDEVICE_EXTENSION DeviceExtension,
    IN UCHAR Byte
    );

NTSTATUS
I8xPutBytePolled(
    IN CCHAR PortType,
    IN BOOLEAN WaitForAcknowledge,
    IN CCHAR AckDeviceType,
    IN PDEVICE_EXTENSION DeviceExtension,
    IN UCHAR Byte
    );

NTSTATUS
I8xPutControllerCommand(
    IN PDEVICE_EXTENSION DeviceExtension,
    IN UCHAR Byte
    );

VOID
I8xServiceParameters(
    IN PINIT_EXTENSION InitializationData,
    IN PUNICODE_STRING RegistryPath,
    IN PUNICODE_STRING KeyboardDeviceName,
    IN PUNICODE_STRING PointerDeviceName
    );

VOID
I8xSetDataQueuePointer(
    IN PVOID Context
    );

VOID
I8xTransmitControllerCommand(
    IN PDEVICE_EXTENSION DeviceExtension,
    IN PVOID Context
    );

BOOLEAN
I8xWriteDataToKeyboardQueue(
    IN PPORT_KEYBOARD_EXTENSION KeyboardExtension,
    IN PKEYBOARD_INPUT_DATA InputData
    );

BOOLEAN
I8xWriteDataToMouseQueue(
    IN PPORT_MOUSE_EXTENSION MouseExtension,
    IN PMOUSE_INPUT_DATA InputData
    );

NTSTATUS
I8xFindWheelMouse(
    IN PDEVICE_OBJECT DeviceObject
    );

VOID
I8xQueueCurrentInput(
    IN PDEVICE_OBJECT DeviceObject
    );

#if defined(WINDOWS_FE) && defined(_X86_)
VOID
I8xServiceCrashDump(
    IN PDEVICE_EXTENSION DeviceExtension,
    IN PUNICODE_STRING RegistryPath,
    IN PUNICODE_STRING KeyboardDeviceName,
    IN PUNICODE_STRING PointerDeviceName
    );
#endif // defined(WINDOWS_FE) && defined(_X86_)

#if defined(NEC_98)
//
// define the scancodes.
//
#define CTRL_KEY                0x1d
#define SHIFT_KEY               0x2a
#define CAPS_KEY                0x3a
#define COPY_KEY                0x37
#define PRINTSCREEN_KEY         0x37
#define ALT_KEY                 0x38
#define F1_KEY                  0x3b
#define F10_KEY                 0x44
#define PAUSE_KEY               0x45
#define NUMLOCK_KEY             0x45  // +E0
#define STOP_KEY                0x46  // +E0
#define SCROLLLOCK_KEY          0x46
#define HOME_KEY                0x47
#define INS_KEY                 0x52
#define VF1_KEY                 0x57
#define VF2_KEY                 0x58
#define VF3_KEY                 0x5D
#define VF4_KEY                 0x5E
#define VF5_KEY                 0x5F
#define KANA_KEY                0x70

//
// Defined the keyboard LED Status.
//
#define KEYBOARD_KANA_LOCK_ON 8
#define KEYBOARD_PC98_LAPTOP_NUMKEY_ON	1

//
//  Keyboard type identifier.
//
#define PC98_106KEY             0x00000000
#define PC98_NmodeKEY           0x00000000
#define PC98_HmodeKEY           0x00000000
#define PC98_LaptopKEY          0x00000000
#define PC98_PTOSKEY            0x00000000
#define PC98_Win95KEY           0x00000001
#define PC98_Win95NOTEKEY       0x00000001
#define PC98_N106KEY            0x00000002

//
// PC9800 specific function prototypes.
//
NTSTATUS
NEC98_KeyboardCommandByte(
    IN PDEVICE_EXTENSION DeviceExtension,
    IN UCHAR KeyboardCommand
    );

NTSTATUS
NEC98_QueueEmulationKey(
    IN PDEVICE_OBJECT deviceObject,
    IN PKEYBOARD_INPUT_DATA input
    );

VOID
NEC98_KeyboardReset(
    IN PDEVICE_EXTENSION DeviceExtension
    );

NTSTATUS
NEC98_TranslateScancodeNEC98(
    IN UCHAR *ScanCode,
    IN USHORT *InputFlags
    );

NTSTATUS
NEC98_TranslateScancodeN106(
    IN UCHAR *ScanCode,
    IN USHORT *InputFlags
    );

VOID
NEC98_PauseKeyEmulation(
    IN PDEVICE_OBJECT deviceObject,
    IN PKEYBOARD_INPUT_DATA input
    );

ULONG
NEC98_KeyboardDetection(
    PDEVICE_EXTENSION deviceExtension
    );

//
// PC-9800 Keyboard ScanCode --> PC/AT 106 Keyboard ScanCode Set
//
static UCHAR ScanCodeSet1_98[128] = {
                0x01,                // 00 ESC
                0x02,                // 01 1
                0x03,                // 02 2
                0x04,                // 03 3
                0x05,                // 04 4
                0x06,                // 05 5
                0x07,                // 06 6
                0x08,                // 07 7
                0x09,                // 08 8
                0x0a,                // 09 9
                0x0b,                // 0A 0
                0x0c,                // 0B -
                0x0d,                // 0C ^                 VK_OEM_7
                0x7d,                // 0D \                 VK_OEM_5
                0x0e,                // 0E BS
                0x0f,                // 0F TAB

                0x10,                // 10 Q
                0x11,                // 11 W
                0x12,                // 12 E
                0x13,                // 13 R
                0x14,                // 14 T
                0x15,                // 15 Y
                0x16,                // 16 U
                0x17,                // 17 I
                0x18,                // 18 O
                0x19,                // 19 P
                0x1a,                // 1A @                 VK_OEM_3
                0x1b,                // 1B [                 VK_OEM_4
                0x1c,                // 1C Enter
                0x1e,                // 1D A
                0x1f,                // 1E S
                0x20,                // 1F D

                0x21,                // 20 F
                0x22,                // 21 G
                0x23,                // 22 H
                0x24,                // 23 J
                0x25,                // 24 K
                0x26,                // 25 L
                0x27,                // 26 ;                 VK_OEM_PLUS
                0x28,                // 27 :                 VK_OEM_1
                0x2b,                // 28 }                 VK_OEM_6
                0x2c,                // 29 Z
                0x2d,                // 2A X
                0x2e,                // 2B C
                0x2f,                // 2C V
                0x30,                // 2D B
                0x31,                // 2E N
                0x32,                // 2F M

                0x33,                // 30 ,                 VK_OEM_COMMA
                0x34,                // 31 .                 VK_OEM_PERIOD
                0x35,                // 32 /                 VK_OEM_2
                0x73,                // 33 _                 VK_OEM_8
                0x39,                // 34 Space             VK_SPACE
                0x79,                // 35 XFER              VK_KANJI
                0x51,                // 36 ROLL UP           VK_NEXT
                0x49,                // 37 ROLL DOWN         VK_PRIOR
                0x52,                // 38 INS               VK_INSERT
                0x53,                // 39 DEL               VK_DELETE
                0x48,                // 3A Up Arrow          VK_UP
                0x4B,                // 3B Left Arrow        VK_LEFT
                0x4D,                // 3C Rihgt Arrow       VK_RIGHT
                0x50,                // 3D Down Arrow        VK_DOWN
                0x47,                // 3E CLR               VK_CLEAR
                0x4f,                // 3F HELP              VK_END

                0x4a,                // 40 -(Num)            VK_SUBTRACT
                0x35,                // 41 /(Num)            VK_DIVIDE
                0x47,                // 42 7(Num)
                0x48,                // 43 8(Num)
                0x49,                // 44 9(Num)
                0x37,                // 45 *(Num)            VK_MULTIPRY
                0x4b,                // 46 4(Num)
                0x4c,                // 47 5(Num)
                0x4d,                // 48 6(Num)
                0x4e,                // 49 +(Num)            VK_ADD
                0x4f,                // 4A 1(Num)
                0x50,                // 4B 2(Num)
                0x51,                // 4C 3(Num)
                0x59,                // 4D =(Num)            VK_OEM_EQU
                0x52,                // 4E 0(Num)
                0x5c,                // 4F ,(Num)            VK_SEPARATER

                0x53,                // 50 .(Num)            VK_DELETE
                0x7b,                // 51 NFER
                0x57,                // 52 vf1
                0x58,                // 53 vf2
                0x5d,                // 54 vf3               VK_F13
                0x5e,                // 55 vf4               VK_F14
                0x5f,                // 56 vf5               VK_F15
                0xff,                // 57 none
                0xff,                // 58 none
                0xff,                // 59 none
                0xff,                // 5A none
                0xff,                // 5B none
                0xff,                // 5C none
                0xff,                // 5D none
                0xff,                // 5E none
                0xff,                // 5F none

                0x46,                // 60 STOP              VK_CANCEL
                0x37,                // 61 COPY              VK_SNAPSHOT
                0x3b,                // 62 f1
                0x3c,                // 63 f2
                0x3d,                // 64 f3
                0x3e,                // 65 f4
                0x3f,                // 66 f5
                0x40,                // 67 f6
                0x41,                // 68 f7
                0x42,                // 69 f8
                0x43,                // 6A f9
                0x44,                // 6B f10
                0x5e,                // 6C Power             VK_POWER
                0x5f,                // 6D Sleep             VK_SLEEP
                0xff,                // 6E none
                0x60,                // 6F Wake              VK_WAKE

                0x2a,                // 70 SHIFT             VK_SHIFT
                0x3a,                // 71 CAPS              VK_CAPITAL
                0x70,                // 72 KANA              VK_KANA
                0x38,                // 73 GRPH              VK_MENU
                0x1d,                // 74 CTRL              VK_CONTROL
                0xff,                // 75 none
                0xff,                // 76 none
                0x5b,                // 77 Left Windows      VK_LWIN
                0x5c,                // 78 Right Windows     VK_RWIN
                0x5d,                // 79 Apprication       VK_APP
                0xff,                // 7A none
                0xff,                // 7B none
                0xff,                // 7C none
                0x36,                // 7D SHIFT(Right)      VK_RSHIFT
                0xff,                // 7E none
                0xff                 // 7F none
};

//
// E0 Flags conversion
//
static UCHAR Ext0FlgNEC98[128] = {
    0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  // 0x00 - 0x0f
    0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  // 0x10 - 0x1f
    0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  // 0x20 - 0x2f
    0, 0, 0, 0, 0, 0, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,  // 0x30 - 0x3f
    0, 1, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  // 0x40 - 0x4f
    0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 1, 0, 0, 0,  // 0x50 - 0x5f
    1, 1, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 1, 1, 0, 1,  // 0x60 - 0x6f
    0, 0, 0, 0, 0, 0, 0, 1,  1, 1, 0, 0, 0, 0, 0, 0,  // 0x70 - 0x7f
};

//
// PC-9800 106 Keyboard ScanCode --> PC/AT 106 Keyboard ScanCode Set
//
static UCHAR ScanCodeSet1_N106[128] = {
                0x01,                // 00 ESC
                0x02,                // 01 1
                0x03,                // 02 2
                0x04,                // 03 3
                0x05,                // 04 4
                0x06,                // 05 5
                0x07,                // 06 6
                0x08,                // 07 7
                0x09,                // 08 8
                0x0a,                // 09 9
                0x0b,                // 0A 0
                0x0c,                // 0B -
                0x0d,                // 0C ^                 VK_OEM_7
                0x7d,                // 0D \                 VK_OEM_5
                0x0e,                // 0E BS
                0x0f,                // 0F TAB

                0x10,                // 10 Q
                0x11,                // 11 W
                0x12,                // 12 E
                0x13,                // 13 R
                0x14,                // 14 T
                0x15,                // 15 Y
                0x16,                // 16 U
                0x17,                // 17 I
                0x18,                // 18 O
                0x19,                // 19 P
                0x1a,                // 1A @                 VK_OEM_3
                0x1b,                // 1B [                 VK_OEM_4
                0x1c,                // 1C Enter
                0x1e,                // 1D A
                0x1f,                // 1E S
                0x20,                // 1F D

                0x21,                // 20 F
                0x22,                // 21 G
                0x23,                // 22 H
                0x24,                // 23 J
                0x25,                // 24 K
                0x26,                // 25 L
                0x27,                // 26 ;                 VK_OEM_PLUS
                0x28,                // 27 :                 VK_OEM_1
                0x2b,                // 28 }                 VK_OEM_6
                0x2c,                // 29 Z
                0x2d,                // 2A X
                0x2e,                // 2B C
                0x2f,                // 2C V
                0x30,                // 2D B
                0x31,                // 2E N
                0x32,                // 2F M

                0x33,                // 30 ,                 VK_OEM_COMMA
                0x34,                // 31 .                 VK_OEM_PERIOD
                0x35,                // 32 /                 VK_OEM_2
                0x73,                // 33 _                 VK_OEM_8
                0x39,                // 34 Space             VK_SPACE
                0x79,                // 35 Maekouho/Henkan   VK_KANJI
                0x51,                // 36 PageDown          VK_NEXT
                0x49,                // 37 PageUp            VK_PRIOR
                0x52,                // 38 Insert            VK_INSERT
                0x53,                // 39 Delete            VK_DELETE
                0x48,                // 3A Up Arrow          VK_UP
                0x4B,                // 3B Left Arrow        VK_LEFT
                0x4D,                // 3C Rihgt Arrow       VK_RIGHT
                0x50,                // 3D Down Arrow        VK_DOWN
                0x47,                // 3E Home              VK_CLEAR
                0x4f,                // 3F End               VK_END

                0x4a,                // 40 -(Num)            VK_SUBTRACT
                0x35,                // 41 /(Num)            VK_DIVIDE
                0x47,                // 42 7(Num)
                0x48,                // 43 8(Num)
                0x49,                // 44 9(Num)
                0x37,                // 45 *(Num)            VK_MULTIPRY
                0x4b,                // 46 4(Num)
                0x4c,                // 47 5(Num)
                0x4d,                // 48 6(Num)
                0x4e,                // 49 +(Num)            VK_ADD
                0x4f,                // 4A 1(Num)
                0x50,                // 4B 2(Num)
                0x51,                // 4C 3(Num)
                0xff,                // 4D none
                0x52,                // 4E 0(Num)
                0xff,                // 4F none

                0x53,                // 50 .                 VK_DELETE
                0x7b,                // 51 Muhenkan
                0x57,                // 52 F11
                0x58,                // 53 F12
                0xff,                // 54 none
                0xff,                // 55 none
                0xff,                // 56 none
                0xff,                // 57 none
                0xff,                // 58 none
                0xff,                // 59 none
                0xff,                // 5A none
                0x45,                // 5B Numlock
                0x1c,                // 5C Enter(Num)
                0x46,                // 5D ScrollLock
                0xff,                // 5E none
                0x29,                // 5F Zenkaku-Hankaku

                0x45,                // 60 Pause             VK_PAUSE
                0x37,                // 61 Print Scrn        VK_SNAPSHOT
                0x3b,                // 62 F1
                0x3c,                // 63 F2
                0x3d,                // 64 F3
                0x3e,                // 65 F4
                0x3f,                // 66 F5
                0x40,                // 67 F6
                0x41,                // 68 F7
                0x42,                // 69 F8
                0x43,                // 6A F9
                0x44,                // 6B F10
                0xff,                // 6C none
                0xff,                // 6D none
                0xff,                // 6E none
                0xff,                // 6F none

                0x2a,                // 70 Shift(Left)       VK_LSHIFT
                0x3a,                // 71 CapsLock          VK_CAPITAL
                0x70,                // 72 Katakana/Hiragana VK_KANA
                0x38,                // 73 Alt(Left)         VK_MENU
                0x1d,                // 74 Ctrl(Left)        VK_LCONTROL
                0x1d,                // 75 Ctrl(Right)       VK_RCONTROL
                0x38,                // 76 Alt(Right)        VK_RMENU
                0xff,                // 77 none
                0xff,                // 78 none
                0xff,                // 79 none
                0xff,                // 7A none
                0xff,                // 7B none
                0xff,                // 7C none
                0x36,                // 7D Shift(Right)      VK_RSHIFT
                0xff,                // 7E none
                0xff,                // 7F none
};

//
// E0 Flags conversion
//
static UCHAR Ext0FlgN106[128] = {
    0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  // 0x00 - 0x0f
    0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  // 0x10 - 0x1f
    0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  // 0x20 - 0x2f
    0, 0, 0, 0, 0, 0, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,  // 0x30 - 0x3f
    0, 1, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  // 0x40 - 0x4f
    0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 1, 0, 0, 0,  // 0x50 - 0x5f
    1, 1, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,  // 0x60 - 0x6f
    0, 0, 0, 0, 0, 1, 1, 0,  0, 0, 0, 0, 0, 0, 0, 0,  // 0x70 - 0x7f
};
#endif // defined(NEC_98)
#endif // _I8042PRT_
