/*++

Copyright (c) 1996 Microsoft Corporation

Module Name:

    sonydcr.h

Abstract:

    Header file for sonydcr.c 

Author:
    
    Shaun Pierce (shaunp) 25-May-96

Environment:

    Kernel mode only

Revision History:


--*/


//
// Device Extension for our Sony Desktop Camera Driver
//

typedef struct _SONYDCR_EXTENSION {

    //
    // Holds the Device Object of the top of our stack
    //
    PDEVICE_OBJECT BusDeviceObject;

    //
    // Holds the Device Object of our parent (1394 bus driver)
    //
    PDEVICE_OBJECT PhysicalDeviceObject;

    //
    // Holds the current generation count of the bus
    //
    ULONG CurrentGeneration;

    //
    // Holds the value read from the base register (sample read request)
    // 
    ULONG BaseRegister;

    //
    // Holds whether or not we're shutdown
    //
    BOOLEAN bShutdown;

} SONYDCR_EXTENSION, *PSONYDCR_EXTENSION;



//
// Various definitions
//

#define RETRY_COUNT                     2

    
//
// Function prototypes
//

NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT DriverObject,
    IN PUNICODE_STRING RegistryPath
    );

NTSTATUS
SonyDcrPnPAddDevice(
    IN     PDRIVER_OBJECT  DriverObject,
    IN     PDEVICE_OBJECT  PhysicalDeviceObject
    );

NTSTATUS
SonyDcrPnP(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

NTSTATUS
SonyDcrSubmitIrpSynch(
    PSONYDCR_EXTENSION DeviceExtension,
    PIRP Irp,
    PIRB Irb
    );

NTSTATUS
SonyDcrSynchCompletionRoutine(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
    IN PKEVENT Event
    );

//
// External functions
//




