/*++

    Copyright (c) 1996 Microsoft Corporation

Module Name:

    monster.h

Abstract:

    Header file for Diamond Multimedia's Monster ASIC

Author:

    Bryan A. Woodruff (bryanw) 14-Oct-1996
    
Environment:

    Kernel Mode Only
    
Revision History:

    29-Mar-97     kencoope
    
        Changed 'FREEDOM' references to 'MONSTER' references.        

--*/

/*
// Pipe defintions
//
// A pipe is defined as a bulk data transfer path (as opposed to simple
// messaging) between the host and DSP and vice-versa.
//
// The maximum number of paths is 24, limited by the total number of 
// FIFOs available.
*/

#define MONSTER_DEVICE_SAMPLE_RATE      44100
#define MONSTER_MAXNUM_PIPES            24
#define MONSTER_MAXNUM_PIPES_ALLOC_MASK 0x000FFFFF

typedef struct {
    
    USHORT  PingBuffer;
    USHORT  PongBuffer;
    USHORT  Size;

} MONSTER_DSP_PIPE_INFO, *PMONSTER_DSP_PIPE_INFO;

/*
// The following register definitions are defined as 
// offsets from the base.
*/

#define MONSTER_MAXNUM_BTU              48

#define MONSTER_HOST_BTU_BASE_INDEX     0
#define MONSTER_DSP_BTU_BASE_INDEX      24

#define MONSTER_MAXNUM_GPIP             6
#define MONSTER_MAXNUM_FIFO             24

#define MONSTER_REG_INDEX               10
#define MONSTER_REG_DATA_LOWORD         12
#define MONSTER_REG_DATA_HIWORD         14

#define MONSTER_INDEX_AUTO_INCREMENT    0x8000

/*
// IRQ processing
*/

#define MONSTER_REG_IRQ_DSP1_OFFSET     0x3A0
#define MONSTER_REG_IRQ_DSP2_OFFSET     0x3B0

#define MONSTER_REG_IRQ_GROUP           0
#define MONSTER_REG_IRQ_DEVICE          2
#define MONSTER_REG_IRQ_GPIP            4
#define MONSTER_REG_IRQ_BTU             6
#define MONSTER_REG_IRQ_CONTROL         8

/*
// Group interrupts
*/

#define MONSTER_IRQ_GROUP_BTU           0x0001
#define MONSTER_IRQ_GROUP_DEVICE        0x0002
#define MONSTER_IRQ_GROUP_MESSAGE       0x0004
#define MONSTER_IRQ_GROUP_GPIP          0x0008
#define MONSTER_IRQ_GROUP_TIMER         0x0010
#define MONSTER_IRQ_GROUP_MASK          0x001F

#define MONSTER_IRQMASK_GROUP_BTU       (MONSTER_IRQ_GROUP_BTU << 8)
#define MONSTER_IRQMASK_GROUP_DEVICE    (MONSTER_IRQ_GROUP_DEVICE << 8)
#define MONSTER_IRQMASK_GROUP_MESSAGE   (MONSTER_IRQ_GROUP_MESSAGE << 8)
#define MONSTER_IRQMASK_GROUP_GPIP      (MONSTER_IRQ_GROUP_GPIP << 8)
#define MONSTER_IRQMASK_GROUP_TIMER     (MONSTER_IRQ_GROUP_TIMER << 8)

/*
// Peripheral interrupts
*/

#define MONSTER_IRQ_DEVICE_MIDI         0x0001
#define MONSTER_IRQ_DEVICE_GPIO         0x0002
#define MONSTER_IRQ_DEVICE_DEVICE0      0x0004
#define MONSTER_IRQ_DEVICE_DEVICE1      0x0008
#define MONSTER_IRQ_DEVICE_DEVICE2      0x0010
#define MONSTER_IRQ_DEVICE_DEVICE3      0x0020
#define MONSTER_IRQ_DEVICE_DEVICE4      0x0040

#define MONSTER_IRQMASK_DEVICE_MIDI     (MONSTER_IRQ_DEVICE_MIDI << 8)
#define MONSTER_IRQMASK_DEVICE_GPIO     (MONSTER_IRQ_DEVICE_GPIO << 8)
#define MONSTER_IRQMASK_DEVICE_DEVICE0  (MONSTER_IRQ_DEVICE_DEVICE0 << 8)
#define MONSTER_IRQMASK_DEVICE_DEVICE1  (MONSTER_IRQ_DEVICE_DEVICE1 << 8)
#define MONSTER_IRQMASK_DEVICE_DEVICE2  (MONSTER_IRQ_DEVICE_DEVICE2 << 8)
#define MONSTER_IRQMASK_DEVICE_DEVICE3  (MONSTER_IRQ_DEVICE_DEVICE3 << 8)
#define MONSTER_IRQMASK_DEVICE_DEVICE4  (MONSTER_IRQ_DEVICE_DEVICE4 << 8)

/*
// GPIP/BTU interrupts
*/

#define MONSTER_IRQ_GPIP_INVALID        0x0080
#define MONSTER_IRQ_GPIP_INDEX_MASK     0x0003

#define MONSTER_IRQ_BTU_INVALID         0x0080
#define MONSTER_IRQ_BTU_TRANSFER_DONE   0x0040
#define MONSTER_IRQ_BTU_INDEX_MASK      0x003F

/*
// Interrupt control
*/

#define MONSTER_IRQ_CONTROL_HOST_PING   0x0001
#define MONSTER_IRQ_CONTROL_DSP1_PING   0x0002
#define MONSTER_IRQ_CONTROL_DSP2_PING   0x0004
#define MONSTER_IRQ_CONTROL_HOST_RESET  0x0008
#define MONSTER_IRQ_CONTROL_DSP1_RESET  0x0010
#define MONSTER_IRQ_CONTROL_DSP2_RESET  0x0020

/*
// BTU control
*/

#define MONSTER_INDEX_BTU_RAM_BASE          0x0000
#define MONSTER_INDEX_BTU_CONTROL           0x0300
#define MONSTER_INDEX_BTU_HOST_BASE_LINK    (MONSTER_INDEX_BTU_CONTROL + 2)
#define MONSTER_INDEX_BTU_HOST_BEST         (MONSTER_INDEX_BTU_CONTROL + 4)
#define MONSTER_INDEX_BTU_HOST_CURRENT      (MONSTER_INDEX_BTU_CONTROL + 6)
#define MONSTER_INDEX_BTU_HOST_SCAN         (MONSTER_INDEX_BTU_CONTROL + 8)
#define MONSTER_INDEX_BTU_DSP_BEST          (MONSTER_INDEX_BTU_CONTROL + 10)
#define MONSTER_INDEX_BTU_DSP_CURRENT       (MONSTER_INDEX_BTU_CONTROL + 12)
#define MONSTER_INDEX_BTU_DSP_SCAN          (MONSTER_INDEX_BTU_CONTROL + 14)

#define MONSTER_BTU_CONTROL_ENABLE          0x0001
#define MONSTER_BTU_CONTROL_ALWAYS_V4P      0x0002
#define MONSTER_BTU_CONTROL_PULSE_ENABLE    0x0100
#define MONSTER_BTU_CONTROL_RESET           0x0200

#if defined( _NTDDK_ )
#include <pshpack1.h>
#endif

typedef struct _MONSTER_PHYSBTU {

    ULONG   Address;                // 32-bit physical address for this mapping
    USHORT  TransferLength;         // transfer length in bytes for this mapping
    USHORT  LinkDescriptor;         // link descriptor
    USHORT  Control;                // control
    USHORT  Status;                 // status
    ULONG   Reserved;               // reserved
    
} MONSTER_PHYSBTU, *PMONSTER_PHYSBTU;

#if defined( _NTDDK_ )
#include <poppack.h>
#endif

typedef struct _MONSTER_BTU {

    USHORT              Index;      // index
    USHORT              Address;    //
    MONSTER_PHYSBTU     Registers;  // 

} MONSTER_BTU, *PMONSTER_BTU;

#define MONSTER_BTU_SIZE    sizeof( MONSTER_PHYSBTU )

/*
// BTU register offsets
*/

#if defined( _NTDDK_ )

#define MONSTER_BTU_OFFSET_ADDRESS          FIELD_OFFSET( MONSTER_PHYSBTU, Address )
#define MONSTER_BTU_OFFSET_TRANSFERLENGTH   FIELD_OFFSET( MONSTER_PHYSBTU, TransferLength )
#define MONSTER_BTU_OFFSET_LINKDESCRIPTOR   FIELD_OFFSET( MONSTER_PHYSBTU, LinkDescriptor )
#define MONSTER_BTU_OFFSET_CONTROL          FIELD_OFFSET( MONSTER_PHYSBTU, Control )
#define MONSTER_BTU_OFFSET_STATUS           FIELD_OFFSET( MONSTER_PHYSBTU, Status )

#define MONSTER_BTU_RAM_BASE( i ) (MONSTER_INDEX_BTU_RAM_BASE + ((i) * sizeof( MONSTER_PHYSBTU )))

#else
 
/*
// The DSP compiler uses word addressing when we really mean byte addressing.
*/

#define MONSTER_BTU_OFFSET_ADDRESS          0
#define MONSTER_BTU_OFFSET_TRANSFERLENGTH   4
#define MONSTER_BTU_OFFSET_LINKDESCRIPTOR   6
#define MONSTER_BTU_OFFSET_CONTROL          8
#define MONSTER_BTU_OFFSET_STATUS           10

#define MONSTER_BTU_RAM_BASE( i ) (MONSTER_INDEX_BTU_RAM_BASE + ((i) * 16))

#endif


/*
// BTU Address bits (DSP only)
*/

#define MONSTER_BTU_ADDRESS_DATAMEMORY  0x40000000
#define MONSTER_BTU_ADDRESS_DSP2        0x80000000

/*
// BTU Control bits
*/

#define MONSTER_BTU_CONTROL_FIFOSEL     0x001F      /* FIFO selection (5 bits) */
#define MONSTER_BTU_CONTROL_RESERVED    0x0020      /* Reserved                */
#define MONSTER_BTU_CONTROL_WRITE_FIFO  0x0040      /* BTU write to FIFO       */
#define MONSTER_BTU_CONTROL_BURST32     0x0080      /* Burst 32 not 16 bytes   */
#define MONSTER_BTU_CONTROL_BYTE_SWAP   0x0100      /* swap byte order on xfer */
#define MONSTER_BTU_CONTROL_WORD_SWAP   0x0200      /* Swap word order on xfer */
#define MONSTER_BTU_CONTROL_HOST_LINK   0x0400      /* link from host memory   */
#define MONSTER_BTU_CONTROL_DSP1_LINK   0x0800      /* link from DSP1 memory   */
#define MONSTER_BTU_CONTROL_DSP2_LINK   0x0C00      /* link from DSP2 memory   */
#define MONSTER_BTU_CONTROL_LINKDEFINED 0x1000      /* link defined            */
#define MONSTER_BTU_CONTROL_HOST_IRQ    0x2000      /* IRQ Host on completion  */
#define MONSTER_BTU_CONTROL_DSP1_IRQ    0x4000      /* IRQ DSP1 on completion  */
#define MONSTER_BTU_CONTROL_DSP2_IRQ    0x6000      /* IRQ DSP2 on completion  */
#define MONSTER_BTU_CONTROL_DSP_MASTER  0x8000      /* DSP bus master vs. host */

/*
// BTU Status bits
*/

#define MONSTER_BTU_STATUS_ENABLE       0x0001
#define MONSTER_BTU_STATUS_BUSY         0x0002

/*
// GPIP Control
*/

#define MONSTER_INDEX_GPIP_BASE         0x0340

#if defined( _NTDDK_ )
#include <pshpack1.h>
#endif

typedef struct _MONSTER_PHYSGPIP {

    USHORT  Control;
    USHORT  Data;

} MONSTER_PHYSGPIP, *PMONSTER_PHYSGPIP;

#if defined( _NTDDK_ )
#include <poppack.h>
#endif

typedef struct _MONSTER_GPIP {

    USHORT              Index;
    USHORT              Address;
    MONSTER_PHYSGPIP    Registers;

} MONSTER_GPIP, *PMONSTER_GPIP;

#if defined( _NTDDK_ )

#define MONSTER_GPIP_OFFSET_CONTROL FIELD_OFFSET( MONSTER_PHYSGPIP, Control )
#define MONSTER_GPIP_OFFSET_DATA    FIELD_OFFSET( MONSTER_PHYSGPIP, Data )

#define MONSTER_GPIP_BASE( i ) (MONSTER_INDEX_GPIP_BASE + (i * sizeof( MONSTER_PHYSGPIP )))

#else

/*
// The DSP compiler uses word addressing when we really mean byte addressing.
*/

#define MONSTER_GPIP_OFFSET_CONTROL     0
#define MONSTER_GPIP_OFFSET_DATA        2

#define MONSTER_GPIP_BASE( i ) (MONSTER_INDEX_GPIP_BASE + ((i) * 4))

#endif


/*
// GPIP Control bits
*/

#define MONSTER_GPIP_CONTROL_AVAILABLE  0x8000   /* space or data available  */
#define MONSTER_GPIP_CONTROL_ENABLE     0x4000   /* GPIP is enabled          */
#define MONSTER_GPIP_CONTROL_RESET      0x2000   /* Reset GPIP               */
#define MONSTER_GPIP_CONTROL_WIDTH16    0x0200   /* 16-bit access else 8-bit */
#define MONSTER_GPIP_CONTROL_WRITE_FIFO 0x0100   /* GPIP write to FIFO, else */
                                                 /*      GPIP read from FIFO */
#define MONSTER_GPIP_CONTROL_HOST_IRQ   0x0040   /* IRQ Host on avaialable   */
#define MONSTER_GPIP_CONTROL_DSP1_IRQ   0x0080   /* IRQ DSP1 on avaialable   */
#define MONSTER_GPIP_CONTROL_DSP2_IRQ   0x00C0   /* IRQ DSP2 on avaialable   */
#define MONSTER_GPIP_CONTROL_FIFOSEL    0x001F   /* FIFO selection           */

/*
// FIFO Control
*/

#define MONSTER_INDEX_FIFO_RAM_BASE             0x0400
#define MONSTER_INDEX_FIFO_CONTROL_STORE_BASE   0x0700
#define MONSTER_INDEX_FIFO_FLAGS_BASE           0x0780

#if defined( _NTDDK_ )
#include <pshpack1.h>
#endif

typedef struct _MONSTER_PHYSFIFO {

    USHORT  Control;

} MONSTER_PHYSFIFO, *PMONSTER_PHYSFIFO;

#if defined( _NTDDK_ )
#include <poppack.h>
#endif

typedef struct _MONSTER_FIFO {

    USHORT              Index;
    USHORT              Address;
    MONSTER_PHYSFIFO    Registers;

} MONSTER_FIFO, *PMONSTER_FIFO;

#if defined( _NTDDK_ )

#define MONSTER_FIFO_OFFSET_CONTROL FIELD_OFFSET( MONSTER_PHYSFIFO, Control )

#define MONSTER_FIFO_CONTROL_BASE( i ) (MONSTER_INDEX_FIFO_CONTROL_STORE_BASE + ((i) * sizeof( MONSTER_PHYSFIFO )))
#define MONSTER_FIFO_FLAGS_BASE( i ) (MONSTER_INDEX_FIFO_FLAGS_BASE + ((i) * sizeof( MONSTER_PHYSFIFO )))

#else

/*
// The DSP compiler uses word addressing when we really mean byte addressing.
*/

#define MONSTER_FIFO_OFFSET_CONTROL 0

#define MONSTER_FIFO_CONTROL_BASE( i ) (MONSTER_INDEX_FIFO_CONTROL_STORE_BASE + ((i) * 2))
#define MONSTER_FIFO_FLAGS_BASE( i ) (MONSTER_INDEX_FIFO_FLAGS_BASE + ((i) * 2))

/*
// The DSP compiler uses word addressing when we really mean byte addressing.
*/

#endif

#define MONSTER_FIFO_CONTROL_SIZE16         0x0000
#define MONSTER_FIFO_CONTROL_SIZE32         0x0001
#define MONSTER_FIFO_CONTROL_SIZE64         0x0002

/*
// Message control
*/

#define MONSTER_INDEX_MESSAGE_BASE          0x0380

#define MONSTER_INDEX_MESSAGE_CONTROL_HOST  (MONSTER_INDEX_MESSAGE_BASE + 0)
#define MONSTER_INDEX_MESSAGE_DATA_HOST     (MONSTER_INDEX_MESSAGE_BASE + 2)
#define MONSTER_INDEX_MESSAGE_CONTROL_DSP1  (MONSTER_INDEX_MESSAGE_BASE + 4)
#define MONSTER_INDEX_MESSAGE_DATA_DSP1     (MONSTER_INDEX_MESSAGE_BASE + 6)
#define MONSTER_INDEX_MESSAGE_CONTROL_DSP2  (MONSTER_INDEX_MESSAGE_BASE + 8)
#define MONSTER_INDEX_MESSAGE_DATA_DSP2     (MONSTER_INDEX_MESSAGE_BASE + 10)

#define MONSTER_MESSAGE_CONTROL_RESET       0x8000
#define MONSTER_MESSAGE_CONTROL_IRQ_MASK    0x6000
#define MONSTER_MESSAGE_CONTROL_IRQ_DSP2    0x6000
#define MONSTER_MESSAGE_CONTROL_IRQ_DSP1    0x4000
#define MONSTER_MESSAGE_CONTROL_IRQ_HOST    0x2000
#define MONSTER_MESSAGE_CONTROL_PTR         0x000F

/*
// Max length of message is 8 words
*/

#define MONSTER_MESSAGE_MAXLEN              0x0008

/*
// MIDI Control
*/

#define MONSTER_INDEX_MIDI_BASE             0x0390

#define MONSTER_INDEX_MIDI_TRANSMIT        (MONSTER_INDEX_MIDI_BASE + 0)
#define MONSTER_INDEX_MIDI_RECEIVE         (MONSTER_INDEX_MIDI_BASE + 4)

#define MONSTER_MIDI_CONTROL_TX_DISCLOOP    0x0200
#define MONSTER_MIDI_CONTROL_TX_TESTMODE    0x0100
#define MONSTER_MIDI_CONTROL_TX_ENABLE      0x0080
#define MONSTER_MIDI_CONTROL_TX_RESET       0x0040
#define MONSTER_MIDI_CONTROL_TX_FIFOSEL     0x001F

#define MONSTER_MIDI_CONTROL_RX_IRQ_DSP2    0x0300
#define MONSTER_MIDI_CONTROL_RX_IRQ_DSP1    0x0200
#define MONSTER_MIDI_CONTROL_RX_IRQ_HOST    0x0100
#define MONSTER_MIDI_CONTROL_RX_ENABLE      0x0080
#define MONSTER_MIDI_CONTROL_RX_RESET       0x0040
#define MONSTER_MIDI_CONTROL_RX_FIFOSEL     0x001F

/*
// General Control
*/

#define MONSTER_INDEX_GENERAL_CONTROL       0x0320
#define MONSTER_INDEX_SOFTWARE_RESET        (MONSTER_INDEX_GENERAL_CONTROL + 0)
#define MONSTER_INDEX_TIMER_SOURCE_COUNT    (MONSTER_INDEX_GENERAL_CONTROL + 6)
#define MONSTER_INDEX_TIMER_SOURCE_VALUE    (MONSTER_INDEX_GENERAL_CONTROL + 4)
#define MONSTER_INDEX_TIMER0_COUNT          (MONSTER_INDEX_GENERAL_CONTROL + 8)
#define MONSTER_INDEX_TIMER0_CONTROL        (MONSTER_INDEX_GENERAL_CONTROL + 10)
#define MONSTER_INDEX_TIMER_HOST_LATENCY    (MONSTER_INDEX_GENERAL_CONTROL + 12)
#define MONSTER_INDEX_GPIO_MODE             (MONSTER_INDEX_GENERAL_CONTROL + 16)
#define MONSTER_INDEX_GPIO_DATA             (MONSTER_INDEX_GENERAL_CONTROL + 18)
#define MONSTER_INDEX_GPIO_CONTROL          (MONSTER_INDEX_GENERAL_CONTROL + 20)
#define MONSTER_INDEX_GPIO_STATUS           (MONSTER_INDEX_GENERAL_CONTROL + 22)

/*
// Software reset
*/

#define MONSTER_RESET_WAVETABLE             0x0008
#define MONSTER_RESET_CODEC                 0x0004
#define MONSTER_RESET_DSP2                  0x0002
#define MONSTER_RESET_DSP1                  0x0001

/*
// Monster setup macros
*/

#define MONSTER_SETUP_FIFO( fifo, index, control )\
{\
    (fifo)->Index = index;\
    (fifo)->Address = MONSTER_FIFO_CONTROL_BASE( index );\
    (fifo)->Registers.Control = control;\
}

#define MONSTER_SETUP_GPIP( gpip, index, control, fifo )\
{\
    (gpip)->Index = index;\
    (gpip)->Address = MONSTER_GPIP_BASE( index );\
    (gpip)->Registers.Data = 0;\
    (gpip)->Registers.Control = control | ((fifo)->Index & MONSTER_GPIP_CONTROL_FIFOSEL);\
}

#define MONSTER_SETUP_BTU( btu, index, address, length, link, control, status, fifo )\
{\
    (btu)->Index = index;\
    (btu)->Address = MONSTER_BTU_RAM_BASE( index );\
    (btu)->Registers.Address = address;\
    (btu)->Registers.TransferLength = length;\
    (btu)->Registers.LinkDescriptor = link;\
    (btu)->Registers.Control = control | ((fifo)->Index & MONSTER_BTU_CONTROL_FIFOSEL);\
    (btu)->Registers.Status = status;\
    (btu)->Registers.Reserved = 0;\
}

#define MONSTER_SETUP_VBTU( vbtu, address, length, link, control, status, fifo )\
{\
    (vbtu)->Address = address;\
    (vbtu)->TransferLength = length;\
    (vbtu)->LinkDescriptor = link;\
    (vbtu)->Control = control | ((fifo)->Index & MONSTER_BTU_CONTROL_FIFOSEL);\
    (vbtu)->Status = status;\
    (vbtu)->Reserved = 0;\
}


#if defined( _NTDDK_ )

/*
// Host specific setup macros
*/

#define MONSTER_DSP_VBTU_LINK( dc, index )\
    LOWORD( dc->physHostBTULinkPages.LowPart + ((index + DSP_LINK_DESCRIPTORS_INDEX) * sizeof( MONSTER_PHYSBTU )) )

#define MONSTER_HOST_VBTU_LINK( dc, vbtu )\
    LOWORD( dc->physHostBTULinkPages.LowPart + ((PUCHAR) vbtu - (PUCHAR) dc->HostVBTUsVa) )

#endif


/*--------------------------------------------------------------------------*/

/*
// Host link descriptors length is 12288 (768 VBTUs total),
// 32 link descriptors are reserved for the DSP (2 per pipe).
*/

#define HOST_LINK_DESCRIPTORS_BUFFER_LENGTH     12288
#define DSP_LINK_DESCRIPTORS_INDEX              ((HOST_LINK_DESCRIPTORS_BUFFER_LENGTH / sizeof( MONSTER_PHYSBTU )) - (2 * MONSTER_MAXNUM_PIPES))
#define HOST_AVG_LINK_DESCRIPTORS_PER_PIPE      30

/*
// Host messages
*/

/* general debugging message, variable length */

#define HM_DEBUG_STRING     0x0000

/*
// DSP messages
*/

#define DM_ALLOC_PIPE               0x0000
#define DM_SET_FORMAT               0x0001
#define DM_GET_PIPE_INFO            0x0002
#define DM_RESET_PIPE               0x0003
#define DM_FREE_PIPE                0x0004
#define DM_START_PIPE               0x0005
#define DM_PAUSE_PIPE               0x0006
#define DM_RESUME_PIPE              0x0007
#define DM_STOP_PIPE                0x0008

/*--------------------------------------------------------------------------*/

/*
// Host interface specific defines
*/

#if defined( _NTDDK_ )

/*
// Monster register macros
*/

#define WRITE_MONSTER_INDEXED_REG_LOWORD( port, Idx, Data )\
{\
    WIN95COMPAT_WritePortUShort( (PUSHORT)(port + MONSTER_REG_INDEX), Idx );\
    WIN95COMPAT_WritePortUShort( (PUSHORT)(port + MONSTER_REG_DATA_LOWORD), Data );\
}

#define WRITE_MONSTER_INDEXED_REG_HIWORD( port, Idx, Data )\
{\
    WIN95COMPAT_WritePortUShort( (PUSHORT)(port + MONSTER_REG_INDEX), Idx );\
    WIN95COMPAT_WritePortUShort( (PUSHORT)(port + MONSTER_REG_DATA_HIWORD), Data );\
}

#define WRITE_MONSTER_INDEXED_REG_ULONG( port, Idx, Data )\
{\
    WIN95COMPAT_WritePortUShort( (PUSHORT)(port + MONSTER_REG_INDEX), Idx );\
    WIN95COMPAT_WritePortUShort( (PUSHORT)(port + MONSTER_REG_DATA_LOWORD), LOWORD( Data ) );\
    WIN95COMPAT_WritePortUShort( (PUSHORT)(port + MONSTER_REG_DATA_HIWORD), HIWORD( Data ) );\
}

__inline USHORT READ_MONSTER_INDEXED_REG_LOWORD(
    PUCHAR port,
    USHORT Idx
    )
{
    WIN95COMPAT_WritePortUShort( (PUSHORT)(port + MONSTER_REG_INDEX), Idx );\
    return WIN95COMPAT_ReadPortUShort( (PUSHORT)(port + MONSTER_REG_DATA_LOWORD) );
}

__inline USHORT READ_MONSTER_INDEXED_REG_HIWORD(
    PUCHAR port,
    USHORT Idx
    )
{
    WIN95COMPAT_WritePortUShort( (PUSHORT)(port + MONSTER_REG_INDEX), Idx );\
    return WIN95COMPAT_ReadPortUShort( (PUSHORT)(port + MONSTER_REG_DATA_HIWORD) );
}

__inline ULONG READ_MONSTER_INDEXED_REG_ULONG(
    PUCHAR port,
    USHORT Idx
)
{
    WIN95COMPAT_WritePortUShort( (PUSHORT)(port + MONSTER_REG_INDEX), Idx );\
    return MAKELONG( WIN95COMPAT_ReadPortUShort( (PUSHORT)(port + MONSTER_REG_DATA_LOWORD) ),
                     WIN95COMPAT_ReadPortUShort( (PUSHORT)(port + MONSTER_REG_DATA_HIWORD) ) );
}

/*
// ADSP-2181 defines
*/

#define ADSP2181_PAD_SIZE       (16*1024)
#define ADSP2181_CODESPACE_SIZE (48*1024)
#define ADSP2181_DATASPACE_SIZE (32*1024)

#define ADSP2181_REGION_MEM_END 0
#define ADSP2181_REGION_MEM_PA  1
#define ADSP2181_REGION_MEM_PO  2
#define ADSP2181_REGION_MEM_DA  3
#define ADSP2181_REGION_MEM_DO  4
#define ADSP2181_REGION_MEM_BO  5

typedef struct _ADSP2181_IMAGE {
    USHORT  Type;
    USHORT  Address;
    USHORT  MaxDataAddress;
    USHORT  MaxCodeAddress;
    PUCHAR  CodeSpace;
    PUCHAR  DataSpace;
    USHORT  Length;
    
} ADSP2181_IMAGE, *PADSP2181_IMAGE;

#endif

