/*****************************************************************************
 * tables.h - SB16 topology miniport tables
 *****************************************************************************
 * Copyright (c) Microsoft Corporation 1997
 */

#ifndef _SB16TOPO_TABLES_H_
#define _SB16TOPO_TABLES_H_

/*****************************************************************************
 * The topology
 *****************************************************************************
 *
 *  wave>-------VOL---------------------+
 *                                      |
 * synth>-------VOL--+------------------+
 *                   |                  |
 *                   +--SWITCH_2X2--+   |
 *                                  |   |
 *    cd>-------VOL--+--SWITCH----------+
 *                   |              |   |
 *                   +--SWITCH_2X2--+   |
 *                                  |   |
 *   aux>-------VOL--+--SWITCH----------+
 *                   |              |   |
 *                   +--SWITCH_2X2--+   |
 *                                  |   |
 *   mic>--AGC--VOL--+--SWITCH----------+--VOL--BASS--TREBLE--GAIN-->lineout
 *                   |              |
 *                   +--SWITCH_1X2--+-------------------------GAIN-->wavein
 *
 */

/*****************************************************************************
 * PinDataRangesBridge
 *****************************************************************************
 * Structures indicating range of valid format values for bridge pins.
 */
static
KSDATARANGE PinDataRangesBridge[] =
{
   {
      sizeof(KSDATARANGE),
      0,
      0,
      0,
      STATICGUIDOF(KSDATAFORMAT_TYPE_AUDIO),
      STATICGUIDOF(KSDATAFORMAT_SUBTYPE_ANALOG),
      STATICGUIDOF(KSDATAFORMAT_SPECIFIER_NONE)
   }
};

/*****************************************************************************
 * PinDataRangePointersBridge
 *****************************************************************************
 * List of pointers to structures indicating range of valid format values
 * for audio bridge pins.
 */
static
PKSDATARANGE PinDataRangePointersBridge[] =
{
    &PinDataRangesBridge[0]
};

/*****************************************************************************
 * MiniportPins
 *****************************************************************************
 * List of pins.
 */
static
MINIPORT_PIN MiniportPins[] =
{
    {
        SIZEOF_ARRAY(PinDataRangePointersBridge),   // WAVEOUT_SOURCE
        PinDataRangePointersBridge,
        0,
        KSPIN_DATAFLOW_IN,
        0,
        NULL,
        0,
        &KSCATEGORY_AUDIO,
        NULL
    },
    {
        SIZEOF_ARRAY(PinDataRangePointersBridge),   // SYNTH_SOURCE
        PinDataRangePointersBridge,
        0,
        KSPIN_DATAFLOW_IN,
        0,
        NULL,
        0,
        &KSNODETYPE_SYNTHESIZER,
        NULL
    },
    {
        SIZEOF_ARRAY(PinDataRangePointersBridge),   // CD_SOURCE
        PinDataRangePointersBridge,
        0,
        KSPIN_DATAFLOW_IN,
        0,
        NULL,
        0,
        &KSNODETYPE_CD_PLAYER,
        NULL
    },
    {
        SIZEOF_ARRAY(PinDataRangePointersBridge),   // LINEIN_SOURCE
        PinDataRangePointersBridge,
        0,
        KSPIN_DATAFLOW_IN,
        0,
        NULL,
        0,
        &KSNODETYPE_LINE_CONNECTOR,
        NULL
    },
    {
        SIZEOF_ARRAY(PinDataRangePointersBridge),   // MIC_SOURCE
        PinDataRangePointersBridge,
        0,
        KSPIN_DATAFLOW_IN,
        0,
        NULL,
        0,
        &KSNODETYPE_MICROPHONE,
        NULL
    },
    {
        SIZEOF_ARRAY(PinDataRangePointersBridge),   // LINEOUT_DEST
        PinDataRangePointersBridge,
        0,
        KSPIN_DATAFLOW_OUT,
        0,
        NULL,
        0,
        &KSNODETYPE_SPEAKER,
        NULL
    },
    {
        SIZEOF_ARRAY(PinDataRangePointersBridge),   // WAVEIN_DEST
        PinDataRangePointersBridge,
        0,
        KSPIN_DATAFLOW_OUT,
        0,
        NULL,
        0,
        &KSCATEGORY_AUDIO,
        NULL
    }
};

enum
{
    WAVEOUT_SOURCE = 0,
    SYNTH_SOURCE,
    CD_SOURCE,
    LINEIN_SOURCE,
    MIC_SOURCE,
    LINEOUT_DEST,
    WAVEIN_DEST
};

/*****************************************************************************
 * TopologyNodes
 *****************************************************************************
 * List of node identifiers.
 */
static
GUID TopologyNodes[] =
{
    STATICGUIDOF(KSNODETYPE_VOLUME),       // WAVEOUT_VOLUME

    STATICGUIDOF(KSNODETYPE_VOLUME),       // SYNTH_VOLUME
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // SYNTH_WAVEIN_SUPERMIX

    STATICGUIDOF(KSNODETYPE_VOLUME),       // CD_VOLUME
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // CD_LINEOUT_SUPERMIX
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // CD_WAVEIN_SUPERMIX

    STATICGUIDOF(KSNODETYPE_VOLUME),       // LINEIN_VOLUME
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // LINEIN_LINEOUT_SUPERMIX
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // LINEIN_WAVEIN_SUPERMIX

    STATICGUIDOF(KSNODETYPE_AGC),          // MIC_AGC
    STATICGUIDOF(KSNODETYPE_VOLUME),       // MIC_VOLUME
    STATICGUIDOF(KSNODETYPE_MUTE),         // MIC_LINEOUT_MUTE
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // MIX_WAVEIN_SUPERMIX

    STATICGUIDOF(KSNODETYPE_SUM),          // LINEOUT_MIX
    STATICGUIDOF(KSNODETYPE_VOLUME),       // LINEOUT_VOL
    STATICGUIDOF(KSNODETYPE_TONE),         // LINEOUT_TONE
    STATICGUIDOF(KSNODETYPE_VOLUME),       // LINEOUT_GAIN

    STATICGUIDOF(KSNODETYPE_SUM),          // WAVEIN_MIX
    STATICGUIDOF(KSNODETYPE_VOLUME)        // WAVEIN_GAIN
};

/*****************************************************************************
 * TopologyNodeNames
 *****************************************************************************
 * List of node name identifiers.
 *
 * TODO - fill in with correct GUIDs
 */
static
GUID TopologyNodeNames[] =
{
    STATICGUIDOF(KSNODETYPE_VOLUME),       // WAVEOUT_VOLUME

    STATICGUIDOF(KSNODETYPE_VOLUME),       // SYNTH_VOLUME
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // SYNTH_WAVEIN_SUPERMIX

    STATICGUIDOF(KSNODETYPE_VOLUME),       // CD_VOLUME
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // CD_LINEOUT_SUPERMIX
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // CD_WAVEIN_SUPERMIX

    STATICGUIDOF(KSNODETYPE_VOLUME),       // LINEIN_VOLUME
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // LINEIN_LINEOUT_SUPERMIX
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // LINEIN_WAVEIN_SUPERMIX

    STATICGUIDOF(KSNODETYPE_AGC),          // MIC_AGC
    STATICGUIDOF(KSNODETYPE_VOLUME),       // MIC_VOLUME
    STATICGUIDOF(KSNODETYPE_MUTE),         // MIC_LINEOUT_MUTE
    STATICGUIDOF(KSNODETYPE_SUPERMIX),     // MIX_WAVEIN_SUPERMIX

    STATICGUIDOF(KSNODETYPE_SUM),          // LINEOUT_MIX
    STATICGUIDOF(KSNODETYPE_VOLUME),       // LINEOUT_VOL
    STATICGUIDOF(KSNODETYPE_TONE),         // LINEOUT_TONE
    STATICGUIDOF(KSNODETYPE_VOLUME),       // LINEOUT_GAIN

    STATICGUIDOF(KSNODETYPE_SUM),          // WAVEIN_MIX
    STATICGUIDOF(KSNODETYPE_VOLUME)        // WAVEIN_GAIN
};

/*****************************************************************************
 * BaseRegisters
 *****************************************************************************
 * Table of H/W access parameters
 */
static
BYTE BaseRegisters[] =
{
    DSP_MIX_VOICEVOLIDX_L,          // WAVEOUT_VOLUME

    DSP_MIX_FMVOLIDX_L,             // SYNTH_VOLUME
    MIXBIT_SYNTH_WAVEIN_R,          // SYNTH_WAVEIN_SUPERMIX

    DSP_MIX_CDVOLIDX_L,             // CD_VOLUME
    MIXBIT_CD_LINEOUT_R,            // CD_LINEOUT_SUPERMIX
    MIXBIT_CD_WAVEIN_R,             // CD_WAVEIN_SUPERMIX

    DSP_MIX_LINEVOLIDX_L,           // LINEIN_VOLUME
    MIXBIT_LINEIN_LINEOUT_R,        // LINEIN_LINEOUT_SUPERMIX
    MIXBIT_LINEIN_WAVEIN_R,         // LINEIN_WAVEIN_SUPERMIX

    0,                              // MIC_AGC
    DSP_MIX_MICVOLIDX,              // MIC_VOLUME
    0,                              // MIC_LINEOUT_MUTE
    0,                              // MIC_WAVEIN_SUPERMIX

    0,                              // LINEOUT_MIX
    DSP_MIX_MASTERVOLIDX_L,         // LINEOUT_VOL
    DSP_MIX_TREBLEIDX_L,            // LINEOUT_TONE
    DSP_MIX_OUTGAINIDX_L,           // LINEOUT_GAIN

    0,                              // WAVEIN_MIX
    DSP_MIX_INGAINIDX_L             // WAVEIN_GAIN
};

enum
{
    WAVEOUT_VOLUME = 0,
    SYNTH_VOLUME,
    SYNTH_WAVEIN_SUPERMIX,
    CD_VOLUME,
    CD_LINEOUT_SUPERMIX,
    CD_WAVEIN_SUPERMIX,
    LINEIN_VOLUME,
    LINEIN_LINEOUT_SUPERMIX,
    LINEIN_WAVEIN_SUPERMIX,
    MIC_AGC,
    MIC_VOLUME,
    MIC_LINEOUT_MUTE,
    MIC_WAVEIN_SUPERMIX,
    LINEOUT_MIX,
    LINEOUT_VOL,
    LINEOUT_TONE,
    LINEOUT_GAIN,
    WAVEIN_MIX,
    WAVEIN_GAIN
};

/*****************************************************************************
 * ConnectionTable
 *****************************************************************************
 * Table of topology unit connections.
 *
 * Pin numbering is technically arbitrary, but the convention established here
 * is to number a solitary output pin 0 (looks like an 'o') and a solitary
 * input pin 1 (looks like an 'i').  Even destinations, which have no output,
 * have an input pin numbered 1 and no pin 0.
 *
 * Nodes are more likely to have multiple ins than multiple outs, so the more
 * general rule would be that inputs are numbered >=1.  If a node has multiple
 * outs, none of these conventions apply.
 *
 * Nodes have at most one control value.  Mixers are therefore simple summing
 * nodes with no per-pin levels.  Rather than assigning a unique pin to each
 * input to a mixer, all inputs are connected to pin 1.  This is acceptable
 * because there is no functional distinction between the inputs.
 *
 * There are no multiplexers in this topology, so there is no opportunity to
 * give an example of a multiplexer.  A multiplexer should have a single
 * output pin (0) and multiple input pins (1..n).  Its control value is an
 * integer in the range 1..n indicating which input is connected to the
 * output.
 *
 * In the case of connections to pins, as opposed to connections to nodes, the
 * node is identified as KSFILTER_NODE and the pin number identifies the
 * particular filter pin.
 */
static
MINIPORT_CONNECTION MiniportConnections[] =
{   //  FromNode,               FromPin,          ToNode,                 ToPin
    {   KSFILTER_NODE,          WAVEOUT_SOURCE,   WAVEOUT_VOLUME,         1             },
    {   WAVEOUT_VOLUME,         0,                LINEOUT_MIX,            1             },

    {   KSFILTER_NODE,          SYNTH_SOURCE,     SYNTH_VOLUME,           1             },
    {   SYNTH_VOLUME,           0,                LINEOUT_MIX,            2             },
    {   SYNTH_VOLUME,           0,                SYNTH_WAVEIN_SUPERMIX,  1             },
    {   SYNTH_WAVEIN_SUPERMIX,  0,                WAVEIN_MIX,             1             },

    {   KSFILTER_NODE,          CD_SOURCE,        CD_VOLUME,              1             },
    {   CD_VOLUME,              0,                CD_LINEOUT_SUPERMIX,    1             },
    {   CD_LINEOUT_SUPERMIX,    0,                LINEOUT_MIX,            3             },
    {   CD_VOLUME,              0,                CD_WAVEIN_SUPERMIX,     1             },
    {   CD_WAVEIN_SUPERMIX,     0,                WAVEIN_MIX,             2             },

    {   KSFILTER_NODE,          LINEIN_SOURCE,    LINEIN_VOLUME,          1             },
    {   LINEIN_VOLUME,          0,                LINEIN_LINEOUT_SUPERMIX,1             },
    {   LINEIN_LINEOUT_SUPERMIX,0,                LINEOUT_MIX,            4             },
    {   LINEIN_VOLUME,          0,                LINEIN_WAVEIN_SUPERMIX, 1             },
    {   LINEIN_WAVEIN_SUPERMIX, 0,                WAVEIN_MIX,             3             },

    {   KSFILTER_NODE,          MIC_SOURCE,       MIC_AGC,                1             },
    {   MIC_AGC,                0,                MIC_VOLUME,             1             },
    {   MIC_VOLUME,             0,                MIC_LINEOUT_MUTE,       1             },
    {   MIC_LINEOUT_MUTE,       0,                LINEOUT_MIX,            5             },
    {   MIC_VOLUME,             0,                MIC_WAVEIN_SUPERMIX,    1             },
    {   MIC_WAVEIN_SUPERMIX,    0,                WAVEIN_MIX,             4             },

    {   LINEOUT_MIX,            0,                LINEOUT_VOL,            1             },
    {   LINEOUT_VOL,            0,                LINEOUT_TONE,           1             },
    {   LINEOUT_TONE,           0,                LINEOUT_GAIN,           1             },
    {   LINEOUT_GAIN,           0,                KSFILTER_NODE,          LINEOUT_DEST  },

    {   WAVEIN_MIX,             0,                WAVEIN_GAIN,            1             },
    {   WAVEIN_GAIN,            0,                KSFILTER_NODE,          WAVEIN_DEST   }
};

/*****************************************************************************
 * Properties
 *****************************************************************************
 * Properties.
 */
static
PROPERTY_ITEM Properties[] =
{
    { 
        &KSPROPSETID_Audio, 
        KSPROPERTY_AUDIO_MUTE,
        KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_SET,
        PropertyHandler_OnOff
    },
    { 
        &KSPROPSETID_Audio, 
        KSPROPERTY_AUDIO_AGC,
        KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_SET,
        PropertyHandler_OnOff
    },
    { 
        &KSPROPSETID_Audio, 
        KSPROPERTY_AUDIO_VOLUMELEVEL,
        KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_SET,
        PropertyHandler_Level
    },
    { 
        &KSPROPSETID_Audio, 
        KSPROPERTY_AUDIO_BASS,
        KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_SET,
        PropertyHandler_Level
    },
    { 
        &KSPROPSETID_Audio, 
        KSPROPERTY_AUDIO_TREBLE,
        KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_SET,
        PropertyHandler_Level
    },
    { 
        &KSPROPSETID_Audio, 
        KSPROPERTY_AUDIO_MIX_LEVEL_CAPS,
        KSPROPERTY_TYPE_GET,
        PropertyHandler_SuperMixCaps
    },
    { 
        &KSPROPSETID_Audio, 
        KSPROPERTY_AUDIO_MIX_LEVEL_TABLE,
        KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_SET,
        PropertyHandler_SuperMixTable
    }
};

/*****************************************************************************
 * MiniportDescription
 *****************************************************************************
 * Complete miniport description.
 */
static
MINIPORT_DESCRIPTION MiniportDescription =
{
    SIZEOF_ARRAY(MiniportPins),
    MiniportPins,
    SIZEOF_ARRAY(TopologyNodes),
    TopologyNodes,
    TopologyNodeNames,
    SIZEOF_ARRAY(MiniportConnections),
    MiniportConnections,
    SIZEOF_ARRAY(Properties),
    Properties,
    sizeof(PROPERTY_ITEM)
};

#endif
