/*
Copyright (c) 1995-1997  Microsoft Corporation

Module Name:
    hidgame.h

Abstract:
    Definitions of all constants and types for the joystick driver.

Author:
        Eliyas Yakub
*/

#ifndef __HIDGAME_H__
#define __HIDGAME_H__


#include <wdm.h>
#include "hidtoken.h"
#include "hidusage.h"
#include "hidport.h"
#include "gameport.h"

//
// BUGBUG BUGBUG BUGBUG
// fill in the read vendor id and product numbers here
// BUGBUG BUGBUG BUGBUG
//
#define HIDGAME_VENDOR_ID       ((USHORT) 'SM')
#define HIDGAME_PRODUCT_ID      ((USHORT) 'RK')
#define HIDGAME_VERSION_NUMBER  ((USHORT) 0)


// Device specific bitmasks
#define X_AXIS_BITMASK  0x01

// Analog joystick bitmasks
#define JOYSTICK2_BUTTON2   0x80
#define JOYSTICK2_BUTTON1   0x40
#define JOYSTICK1_BUTTON2   0x20
#define JOYSTICK1_BUTTON1   0x10
#define JOYSTICK2_Y_MASK    0x08
#define JOYSTICK2_X_MASK    0x04
#define JOYSTICK1_R_MASK    0x08
#define JOYSTICK1_Z_MASK    0x04
#define JOYSTICK1_Y_MASK    0x02
#define JOYSTICK1_X_MASK    0x01

#define JOYSTICKID1                     0 //defined in mmsystem.h
#define JOYSTICKID2                     1

#define JOY_START_TIMERS    0

// Device specific timer values

// 16 mS upper bound on analog polling,
// 8ms largest expected value, use 16 for safety
#define ANALOG_POLL_TIMEOUT    16000

// 100 uS accuracy on polling time
#define ANALOG_POLL_RESOLUTION   100

// Retry timeout interval  in ms
// should be half of the ANALOG_POLL_TIMEOUT  value
#define HGAME_RETRY_INTERVAL 8000


#define HGAME_NUMBER_DESCRIPTORS 1

typedef struct _HIDGAME_INPUT_DATA {

    union {
        struct _HIDGAME_HALF_STICK {
            struct _HIDGAME_STICK {
                ULONG   Button1 : 1;  // Secondary button
                ULONG   X       : 31;
                ULONG   Button2 : 1;
                ULONG   Y       : 31;
            } HalfStick;
        };

        struct _HIDGAME_1STICK {
            ULONG       Button1 : 1;  // Primary button
            ULONG       X       : 31;
            ULONG       Button2 : 1;
            ULONG       Y       : 31;
            ULONG       Button3 : 1;
            ULONG       Z       : 31;  // Rotation
            ULONG       Button4 : 1;
            ULONG       T       : 31;  // Throttle
        };
    };
} HIDGAME_INPUT_DATA, *PHIDGAME_INPUT_DATA;


#define GAME_1STICK_REPORT \
    {   HIDP_GLOBAL_USAGE_PAGE_1, HID_USAGE_PAGE_GENERIC, \
        HIDP_LOCAL_USAGE_1, HID_USAGE_GENERIC_JOYSTICK, \
        HIDP_GLOBAL_LOG_MAX_4, 0xff, 0xff, 0xff, 0x3f, \
        HIDP_GLOBAL_LOG_MIN_4, 0xff, 0xff, 0xff, 0x7f, \
        HIDP_MAIN_COLLECTION, 0x00, \
            HIDP_GLOBAL_REPORT_COUNT_1, 0x01, \
\
            HIDP_GLOBAL_REPORT_SIZE, 0x1, \
            HIDP_LOCAL_USAGE_4, 0x1, 0x0, HID_USAGE_PAGE_BUTTON, 0x0,\
            HIDP_MAIN_INPUT_1, 0X02, \
            HIDP_GLOBAL_REPORT_SIZE, 0X1F, \
            HIDP_LOCAL_USAGE_4, HID_USAGE_GENERIC_X, 0x0, HID_USAGE_PAGE_GENERIC, 0x0,\
            HIDP_MAIN_INPUT_1, 0X02, \
\
            HIDP_GLOBAL_REPORT_SIZE, 0x1, \
            HIDP_LOCAL_USAGE_4, 0x2, 0x0, HID_USAGE_PAGE_BUTTON, 0x0,\
            HIDP_MAIN_INPUT_1, 0X02, \
            HIDP_GLOBAL_REPORT_SIZE, 0X1F, \
            HIDP_LOCAL_USAGE_4, HID_USAGE_GENERIC_Y, 0x0, HID_USAGE_PAGE_GENERIC, 0x0, \
            HIDP_MAIN_INPUT_1, 0X02, \
\
            HIDP_GLOBAL_REPORT_SIZE, 0x1, \
            HIDP_LOCAL_USAGE_4, 0x3, 0x0, HID_USAGE_PAGE_BUTTON, 0x0, \
            HIDP_MAIN_INPUT_1, 0X02, \
            HIDP_GLOBAL_REPORT_SIZE, 0X1F, \
            HIDP_LOCAL_USAGE_4, HID_USAGE_GENERIC_Z, 0x0, HID_USAGE_PAGE_GENERIC, 0x0,\
            HIDP_MAIN_INPUT_1, 0X02, \
\
            HIDP_GLOBAL_REPORT_SIZE, 0x1, \
            HIDP_LOCAL_USAGE_4, 0x4, 0x0, HID_USAGE_PAGE_BUTTON, 0x0,\
            HIDP_MAIN_INPUT_1, 0X02, \
            HIDP_GLOBAL_REPORT_SIZE, 0X1F, \
            HIDP_LOCAL_USAGE_4, HID_USAGE_VEHICLE_THROTTLE, 0x0, HID_USAGE_PAGE_VEHICLE, 0x0,\
            HIDP_MAIN_INPUT_1, 0X02,\
\
        HIDP_MAIN_ENDCOLLECTION \
    }

#define GAME_HALF_STICK_REPORT \
{   HIDP_GLOBAL_USAGE_PAGE_1, HID_USAGE_PAGE_GENERIC, \
    HIDP_LOCAL_USAGE_1, HID_USAGE_GENERIC_JOYSTICK, \
    HIDP_GLOBAL_LOG_MAX_4, 0xff, 0xff, 0xff, 0x3f, \
    HIDP_GLOBAL_LOG_MIN_4, 0xff, 0xff, 0xff, 0x7f, \
    HIDP_MAIN_COLLECTION, 0x00,\
        HIDP_GLOBAL_REPORT_COUNT_1, 0x01, \
\
        HIDP_GLOBAL_REPORT_SIZE, 0x1, \
        HIDP_LOCAL_USAGE_4, 0x1, 0x0, HID_USAGE_PAGE_BUTTON, 0x0,\
        HIDP_MAIN_INPUT_1, 0X02, \
        HIDP_GLOBAL_REPORT_SIZE, 0X1F, \
        HIDP_LOCAL_USAGE_4, HID_USAGE_GENERIC_X, 0x0, HID_USAGE_PAGE_GENERIC, 0x0, \
        HIDP_MAIN_INPUT_1, 0X02, \
\
        HIDP_GLOBAL_REPORT_SIZE, 0x1, \
        HIDP_LOCAL_USAGE_4, 0x2, 0x0, HID_USAGE_PAGE_BUTTON, 0x0,\
        HIDP_MAIN_INPUT_1, 0X02, \
        HIDP_GLOBAL_REPORT_SIZE, 0X1F, \
        HIDP_LOCAL_USAGE_4, HID_USAGE_GENERIC_Y, 0x0, HID_USAGE_PAGE_GENERIC, 0x0, \
        HIDP_MAIN_INPUT_1, 0X02, \
\
        HIDP_GLOBAL_REPORT_SIZE, 0x40, \
        HIDP_GLOBAL_REPORT_COUNT_1, 0x01, \
        HIDP_MAIN_INPUT_1, 0x03,\
    HIDP_MAIN_ENDCOLLECTION \
}



// These stats are used for performance testing and debugging
typedef struct
{
    ULONG   Polls;
    ULONG   Timeouts;
    ULONG   Frequency;
    ULONG   dwQPCLatency;
    LONG    nQuiesceLoop;
    ULONG   Version;
    ULONG   PolledTooSoon;
    ULONG   NumberOfAxes;
    BOOLEAN    bTwoSticks;
    ULONG   Redo;
} JOY_STATISTICS, *PJOY_STATISTICS;

// A global struct to store driver wide global data
typedef struct _HIDGAME_GLOBAL
{
    // Keeps list of all the devices
    LIST_ENTRY DeviceListHead;

} HIDGAME_GLOBAL;

// Device extension data
typedef struct _DEVICE_EXTENSION{

    //Hid descriptor for this device
    HID_DESCRIPTOR HidDescriptor;

    // game resource info structure filled by GameEnumerator
    GAMEENUM_PORT_PARAMETERS PortInfo;

    // time at which the joystick was last polled
    LARGE_INTEGER LastPollTime;   // set whenever the joystick's polled

    // A Spinlock is used to synchronize access to this device. This is
    // a pointer to the actual spinlock data area
    PKSPIN_LOCK pSpinLock;

    // Actual SpinLock data area
    KSPIN_LOCK SpinLock;

    // doubly linked list to keep a list of devices
    LIST_ENTRY Link;

    //Polling timer object
    KTIMER  PollingTimer;

    //Polling interval (retry interval)
    LARGE_INTEGER PollingInterval;

    //Irps waiting for processing are queued in this object
    KDEVICE_QUEUE    PendingIrpQueue;

    //Customtimer dpc object
    KDPC PollingDpc;

    //status of joystick
    BOOLEAN                 Unplugged;

    // The high resolution system clock (from KeQueryPerformanceCounter)
    // is updated at this frequency
    ULONG Frequency;

    // min number of KeQueryPerformanceCounter ticks between polls
    // Used to prevent too-frequent polling of joystick
    ULONG nMinTicksBetweenPolls;


    // The maximum duration of a polling cycle (expressed in ticks).
    ULONG MaxTimeoutInTicks;

    // The maximum duration of a polling cycle for use in quiesce wait
    LONG nQuiesceLoop;

    // The minimum resolution of a polling cycle. This is used to detect
    // if we've been pre-empted or interrupted during a polling loop. If
    // we have been, we can retry the operation.
    ULONG ThresholdInTicks;

    // This device has been started
    BOOLEAN                 Started;

    // This device has been removed
    BOOLEAN                 Removed;

    UCHAR                   Reseved2[1];

    // a back pointer to the actual DeviceObject
    PDEVICE_OBJECT          Self;

    // The PDO to which this function driver is attached
    //PDEVICE_OBJECT          PDO;

    // The top of the device stack just beneath this  device object.
    PDEVICE_OBJECT          TopOfStack;

    // an event to sync the start IRP.
    KEVENT                  StartEvent;

    // an event to synch outstandIO to zero
    KEVENT                  RemoveEvent;

    // 1 biased count of reasons why this object should stick around
    ULONG                   OutstandingIO;

    // Debugging and performance testing
    JOY_STATISTICS          JoyStatistics;

}  DEVICE_EXTENSION, *PDEVICE_EXTENSION;

#define GET_MINIDRIVER_DEVICE_EXTENSION(DO)  \
    ((PDEVICE_EXTENSION) (((PHID_DEVICE_EXTENSION)(DO)->DeviceExtension)->MiniDeviceExtension))

#define GET_NEXT_DEVICE_OBJECT(DO) \
    (((PHID_DEVICE_EXTENSION)(DO)->DeviceExtension)->NextDeviceObject)


//  Debugging macros

#ifdef DEBUG
#define HGAME_KdPrint(_x_) \
                DbgPrint("HIDGAME.SYS: ");\
                DbgPrint _x_;

#define TRAP()  DbgBreakPoint()

#else

#define HGAME_KdPrint(_x_)
#define TRAP()


#endif



NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT  DriverObject,
    IN PUNICODE_STRING registryPath
);

NTSTATUS
HGAME_CreateClose (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP pIrp
    );

NTSTATUS
HGAME_Power (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP pIrp
    );

NTSTATUS
HGAME_PnP (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP pIrp
    );

NTSTATUS
HGAME_Internal_Ioctl (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP pIrp
    );


NTSTATUS
HGAME_AddDevice(
    IN PDRIVER_OBJECT DriverObject,
    IN PDEVICE_OBJECT PhysicalDeviceObject
    );


VOID
HGAME_Unload(
    IN PDRIVER_OBJECT DriverObject
    );

NTSTATUS
HGAME_StartDevice (
    IN PDEVICE_EXTENSION    pDevExt,
    IN PIRP                 pIrp
    );


VOID
HGAME_StopDevice (
    IN PDEVICE_EXTENSION    pDevExt,
    IN BOOLEAN              TouchTheHardware
    );


NTSTATUS
HGAME_PnPComplete (
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP pIrp,
    IN PVOID Context
    );

VOID
HGAME_PollingTimerDpc(
    IN PKDPC Dpc,
    IN PVOID Context,
    IN PVOID SystemArgument1,
    IN PVOID SystemArgument2
);

VOID
HGAME_CancelIrp(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP pIrp
);

NTSTATUS
HGAME_ReadReport(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP pIrp
);

NTSTATUS
HGAME_GetReportDescriptor(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP pIrp
);

NTSTATUS
HGAME_GetDeviceDescriptor(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP pIrp
);

ULONG
TimeInMicroSeconds(
    ULONG dwTime,
    ULONG Frequency
);


ULONG
TimeInTicks(
    ULONG dwTimeInMicroSeconds,
    ULONG Frequency
);


NTSTATUS
HGAME_AnalogPoll(
    PDEVICE_OBJECT DeviceObject,
    PIRP    pIrp
);


NTSTATUS
HGAME_GetResources(
    IN PDEVICE_EXTENSION pDevExt,
    IN  PIRP  pIrp
);

NTSTATUS
HGAME_GetResourcesComplete (
    IN PDEVICE_OBJECT   DeviceObject,
    IN PIRP             pIrp,
    IN PVOID            Context
);

NTSTATUS
HGAME_GetAttributes (
    PDEVICE_OBJECT  DeviceObject,
    PIRP            Irp
    );



#endif // __NTDDJOY_H__
