/*******************************************************************
*
*				 MPINIT.H
*
*				 Copyright (C) 1995 SGS-THOMSON Microelectronics.
*
*
*				 Prototypes for MPINIT.C
*
*******************************************************************/
#ifndef __MPINIT_H__
#define __MPINIT_H__
#define NO_ERROR 0
#ifndef NULL
#define NULL 0
#endif

/*****************************************************************************
*
* The following structures are samples of information that could be used in
* a device extension structure
*
*****************************************************************************/

//
// prototype for the function to send data to a device
//

typedef ULONG (*PFNSEND_DATA)  (PHW_STREAM_REQUEST_BLOCK pSrb,
                                struct _EXTENSION_DATA_HANDLER *pxdhl);

typedef VOID (*PFN_DATASTUB)  (PHW_STREAM_OBJECT pstrm);

typedef struct _EXTENSION_DATA_HANDLER {

	PKSSTREAM_HEADER pPacket;    // pointer to current data packet in process
    PFNSEND_DATA        pfnSendData;// function to send the data to the correct board
	ULONG	            cPacket;	// count of data packets processed
	ULONG	            cOffs;		// offset into the current data packet
    PFN_DATASTUB        pfnDataStub; // timer stub routine
    PHW_STREAM_REQUEST_BLOCK *pCurrentSRB;// current request being processed

} EXTENSION_DATA_HANDLER, *PEXTENSION_DATA_HANDLER;

typedef struct _VIDEO_DEVICE_EXTENSION {

	STREAM_SYSTEM_TIME			videoSTC;   // current video presentation time
	PHW_STREAM_REQUEST_BLOCK  	pCurrentSRB;// current video request in progress
    EXTENSION_DATA_HANDLER      xdhl;       // data handling info for common handler
	PVOID                       pDMABuf;    // pointer to the video DMA buffer
    STREAM_PHYSICAL_ADDRESS     pPhysDmaBuf;// physical address of DMA buffer
    ULONG                       cDmaBuf;    // size of DMA buffer
	KSSTATE                     DeviceState;// current device state
    BOOLEAN                     IRQExpected;// IRQ expected

} VIDEO_DEVICE_EXTENSION, *PVIDEO_DEVICE_EXTENSION;

typedef struct _AUDIO_DEVICE_EXTENSION {

	STREAM_SYSTEM_TIME		    audioSTC;   // current audio presentation time
	PHW_STREAM_REQUEST_BLOCK    pCurrentSRB;// current audio SRB in progress
    EXTENSION_DATA_HANDLER      xdhl;       // data handling info for common handler
	KSSTATE                     DeviceState;// current device state
    BOOLEAN                     IRQExpected;// IRQ expected

} AUDIO_DEVICE_EXTENSION, *PAUDIO_DEVICE_EXTENSION;

//
// definition of the full HW device extension structure This is the structure
// that will be allocated in HW_INITIALIZATION by the stream class driver
// Any information that is used in processing a device request (as opposed to
// a STREAM based request) should be in this structure.  A pointer to this
// structure will be passed in all requests to the minidriver. (See
// HW_STREAM_REQUEST_BLOCK in STRMINI.H)
//

typedef struct _HW_DEVICE_EXTENSION {
	PULONG      ioBaseLocal;                // board base address
	USHORT		Irq;                        // irq level
	PHW_STREAM_REQUEST_BLOCK	AudioQ;     // queues for audio and video request
	PHW_STREAM_REQUEST_BLOCK	VideoQ;     // not used in this model
	VIDEO_DEVICE_EXTENSION 	VideoDeviceExt; // video and audio device extensions
	AUDIO_DEVICE_EXTENSION 	AudioDeviceExt; 
	PHW_STREAM_REQUEST_BLOCK pCurSrb;       // current device request in progress

} HW_DEVICE_EXTENSION, *PHW_DEVICE_EXTENSION;

//
// this structure is our per stream extension structure.  This stores
// information that is relevant on a per stream basis.  Whenever a new stream
// is opened, the stream class driver will allocate whatever extension size
// is specified in the HwInitData.PerStreamExtensionSize.
//

typedef struct _STREAMEX {

    PEXTENSION_DATA_HANDLER pxdhl;

} STREAMEX, *PSTREAMEX;

//
// this structure defines the per request extension.  It defines any storage
// space that the min driver may need in each request packet.
//

typedef struct _SRB_EXTENSION {

    ULONG foo;

} SRB_EXTENSION, * PSRB_EXTENSION;

/*****************************************************************************
*
* the following section defines prototypes for the minidriver initialization
* routines
*
******************************************************************************/

//
// DriverEntry:
//
// This routine is called when the mini driver is first loaded.  The driver
// should then call the StreamClassRegisterAdapter function to register with
// the stream class driver
//

ULONG DriverEntry (PVOID Context1, PVOID Context2);

//
// This routine is called by the mini driver with configuration
// information for an adapter that the mini driver should load on. In response
// to an SRB_INITIALIZE_DEVICE function. The mini
// driver should still perform a small verification to determine that the
// adapter is present at the specified addresses, but should not attempt to
// find an adapter as it would have with previous NT miniports.
//
// All initialization of the adapter should also be performed at this time.
//
// Note: This routine does not have to complete syncronously, and is not
// considered completed until the request has been called back with a completion
// notification.
//

NTSTATUS HwInitialize (IN PHW_STREAM_REQUEST_BLOCK pSrb);

//
// This routine is called when the system is going to remove or disable the
// device.
//
// The mini-driver should free any system resources that it allocated at this
// time.  Note that system resources allocated for the mini-driver by the
// stream class driver will be free'd by the stream driver, and should not be
// free'd in this routine.  (Such as the HW_DEVICE_EXTENSION)
//

BOOLEAN HwUnInitialize ( IN PVOID DeviceExtension);

#endif //__MPINIT_H__

