
/*++

Copyright (c) 1996  Microsoft Corporation

Module Name:

   mpirq.c

Abstract:

   This is the WDM sample streaming class minidriver.  This module contains
   code related to minidriver interrupt processing

Author:

Environment:

   Kernel mode only


Revision History:

--*/

#include "common.h"

/*
** HwInterrupt()
**
**   Routine is called when an interrupt at the IRQ level specified by the
**   ConfigInfo structure passed to the HwInitialize routine is received.
**
**   Note: IRQs may be shared, so the device should ensure the IRQ received
**         was expected
**
** Arguments:
**
**  pHwDevEx - the device extension for the hardware interrupt
**
** Returns:
**
** Side Effects:  none
*/

BOOLEAN HwInterrupt ( IN PHW_DEVICE_EXTENSION  pHwDevEx)
{

BOOL fMyIRQ = FALSE; 

    if (pHwDevEx->VideoDeviceExt.IRQExpected)
    {
        pHwDevEx->VideoDeviceExt.IRQExpected = FALSE;

        //
        // call the video handling IRQ routine here
        //

        fMyIRQ = TRUE;
    }

    if (pHwDevEx->AudioDeviceExt.IRQExpected)
    {
        pHwDevEx->AudioDeviceExt.IRQExpected = FALSE;

        //
        // call the audio handling IRQ routine here
        //

        fMyIRQ = TRUE;
    }

    //
    // returning FALSE indicates that this was not an IRQ for this device, and
    // the IRQ dispatcher will pass the IRQ down the chain to the next handler
    // for this IRQ level
    //

    return(fMyIRQ);
}

