//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE.
//
//  Copyright (c) 1996 Microsoft Corporation.  All Rights Reserved.
//


//
// Dumps the USB Configuration Descriptor
//

#include <windows.h>

#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <time.h>

#include "devioctl.h"

#include "usbdi.h"

#include "ioctl.h"

#define NOISY(_x_) printf _x_ ;

char inPipe[32] = "PIPE00";
char outPipe[32] = "PIPE01";
int PerfMode = 0;
int TestMode = 0;

HANDLE
open_dev(char *devname)
{
	char completeDeviceName[64] = "";
	HANDLE hDEV;

	strcat (completeDeviceName,
            "\\\\.\\"
            );

    strcat (completeDeviceName,
			devname
			);

	if(!PerfMode && !TestMode)
		printf("DeviceName = (%s)\n", completeDeviceName);

	hDEV = CreateFile(completeDeviceName,
		GENERIC_WRITE | GENERIC_READ,
		FILE_SHARE_WRITE | FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		0,
		NULL);

	if (hDEV == INVALID_HANDLE_VALUE) {
		NOISY(("Failed to open (%s) = %d", completeDeviceName, GetLastError()));
	} else {
		if(!PerfMode && !TestMode)
			NOISY(("Opened successfully.\n"));
    }		

	return hDEV;
}

HANDLE
open_file(char *devname, char *filename)
{

	char completeDeviceName[64] = "";
	int success = 1;
	HANDLE h;


	strcat (completeDeviceName,
            "\\\\.\\"
            );

    strcat (completeDeviceName,
			devname
			);

    strcat (completeDeviceName,
			"\\"
			);			

     strcat (completeDeviceName,
			filename
			);					

	if(!PerfMode && !TestMode)
		printf("completeDeviceName = (%s)\n", completeDeviceName);

	h = CreateFile(completeDeviceName,
		GENERIC_WRITE | GENERIC_READ,
		FILE_SHARE_WRITE | FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		0,
		NULL);

	if (h == INVALID_HANDLE_VALUE) {
		NOISY(("Failed to open (%s) = %d", completeDeviceName, GetLastError()));
		success = 0;
	} else {
		if(!PerfMode && !TestMode)
			NOISY(("Opened successfully.\n"));
    }		

	return h;
}

void
usage()
{
    static int i=1;

    if (i) {
        printf("usage:\n");
        printf("-r [n] where n is number of bytes to read\n");
        printf("-w [n] where n is number of bytes to write\n");
        printf("-c [n] where n is number of iterations (default = 1)\n");
        printf("-i [s] where s is the input pipe\n");
        printf("-o [s] where s is the output pipe\n");
        printf("-p perf mode - reports bytes per second throughput\n");
        printf("-s check input data for standard iso pattern\n");
        printf("-t test mode - less screen I/O with pass/fail at end of test\n");
        i = 0;
    }
}

int Count = 1;
int WriteLen = 0;
int ReadLen = 0;
int Iso = 0;

void
parse(
    int argc,
    char *argv[],
    char *devname)
{
    int i;

    for (i=0; i<argc; i++) {
        if (argv[i][0] == '-' ||
            argv[i][0] == '/') {
            switch(argv[i][1]) {
            case 'r':
                ReadLen = atoi(&argv[i+1][0]);
                i++;
                break;
            case 'w':
                WriteLen = atoi(&argv[i+1][0]);
                i++;
                break;
            case 'c':
                Count = atoi(&argv[i+1][0]);
                i++;
                break;
            case 's':
                Iso = 1;
                break;
            case 'p':
                PerfMode = 1;
                break;
            case 't':
                TestMode = 1;
                break;
            case 'i':
                strcpy(inPipe, &argv[i+1][0]);
                i++;
                break;                     
             case 'o':
                strcpy(outPipe, &argv[i+1][0]);
                i++;
                break;                    
            default:
                usage();
            }
        }
    }
}

int 
compare_buffs(char *buff1, char *buff2, int length)
{
    int ok = 1;
    
	if (memcmp(buff1, buff2, length )) {
		DebugBreak();

		// Edi, and Esi point to the mismatching char and ecx indicates the 
		// remaining length.  
		ok = 0;
	}

#if 0
	{
		int i;

	    for (i=0; i<length; i++) {
	        if (buff1[i] != buff2[i]) {

	            _asm {
	                push esi
	                push edi
	                push eax

	                mov esi, buff1
	                mov edi, buff2
	                mov eax, i

	                int 3

	                pop eax
	                pop edi
	                pop esi
	            }
            
    	        ok = 0;
	        }
	    }
	}
#endif
    return ok;
}

void
dump(
   UCHAR *b,
   int len)
{
    int i;

    for (i=0; i<len; i++) {
        printf("%02.2x ", *(b+i));
    }
    printf("\n");
}

int _cdecl main(
    int argc,
	char *argv[])
{
    char *pinBuf = NULL, *poutBuf = NULL;
    int nBytesRead, nBytesWrite, i, j, nBytes;
    int x =0, ok;
    BOOLEAN success;
    HANDLE hRead = INVALID_HANDLE_VALUE, hWrite = INVALID_HANDLE_VALUE;
	char buf[1024];
	clock_t	start, finish;
	ULONG totalBytes = 0L;
	double seconds;
	ULONG fail = 0L;
  
    parse(argc, argv, "I82930-0");

	if ((ReadLen) || (WriteLen)) {

	    if (ReadLen) {
            //
            // open the output file
            //
            hRead = open_file("I82930-0", inPipe);
	    
	        pinBuf = malloc(ReadLen);

	    }

	    if (WriteLen) {

	        hWrite = open_file("I82930-0", outPipe);
	        poutBuf = malloc(WriteLen);
	    }

		// turn on perf timing if enabled
		if(PerfMode)
		{
		    PI82930_INTERFACE_INFO interfaceInfo;
		    PI82930_PIPE_INFO pipeInfo;
			ULONG WriteMPS = 0L, ReadMPS = 0L;
			HANDLE hDev;
			int pipeNum;

			// let's open the device to get pipe info
	        hDev = open_dev("I82930-0");

			// lets get the MPS for these pipes
	        interfaceInfo = (PI82930_INTERFACE_INFO) buf;

			if(hDev != INVALID_HANDLE_VALUE)
			{
				success = DeviceIoControl(hDev,
						IOCTL_I82930_GET_PIPE_INFO,
						buf,
						sizeof(buf),
						buf,
						sizeof(buf),
						&nBytes,
						NULL);


				CloseHandle(hDev);

				// get out pipe MPS
				pipeNum = atoi(outPipe);
			    pipeInfo = &interfaceInfo->Pipes[pipeNum];
				WriteMPS = 	pipeInfo->MaximumPacketSize;

				// get read pipe MPS
				pipeNum  = atoi(inPipe);
			    pipeInfo = &interfaceInfo->Pipes[pipeNum];
				ReadMPS  = 	pipeInfo->MaximumPacketSize;

			}


			printf("\n\nI82930 Perf Stats\n\n");
			printf("Write Pipe MPS = %d     Read Pipe MPS = %d\n", WriteMPS, ReadMPS);
			printf("WriteBufferSize = %d    ReadBufferSize = %d\n", WriteLen, ReadLen);

			if(hWrite != INVALID_HANDLE_VALUE)
			{
				success = DeviceIoControl(hWrite,
							IOCTL_I82930_START_PERF_TIMER,
							buf,
							sizeof(buf),
							buf,
							sizeof(buf),
							&nBytes,
							NULL);
			}

			if(hRead != INVALID_HANDLE_VALUE)
			{
				success = DeviceIoControl(hRead,
							IOCTL_I82930_START_PERF_TIMER,
							buf,
							sizeof(buf),
							buf,
							sizeof(buf),
							&nBytes,
							NULL);
			}
		}

		// strat timing
		if(PerfMode)
			start = clock();

        for (i=0; i<Count; i++) {

    	    if (WriteLen && poutBuf && hWrite != INVALID_HANDLE_VALUE) {

                //
                // put some data in the output buffer
                //

    	        for (j=0; j<WriteLen; j++) {
    	            *(poutBuf+j) = x + Count;
    	            x++;
    	        }

                //
                // send the write
                //

	            WriteFile(hWrite,
	                      poutBuf,
	                      WriteLen,
	                      &nBytesWrite,
	                      NULL);

				// do screen I/O if we aren't in perf mode
				if(!PerfMode && !TestMode)
				{
	                printf("<%s> W (%04.4d) : request %06.6d bytes -- %06.6d bytes written\n", 
	                    outPipe, i, WriteLen, nBytesWrite);
				}
				// take care of perf stuff here
				else
				{
					totalBytes += nBytesWrite;
				}
                assert(nBytesWrite == WriteLen);
	        }

	        if (ReadLen && pinBuf) {

	            success = ReadFile(hRead,
	                          pinBuf,
                              ReadLen,
	                          &nBytesRead,
	                          NULL);

				// do screen I/O if we aren't in perf mode
				if(!PerfMode && !TestMode)
				{
	                printf("<%s> R (%04.4d) : request %06.6d bytes -- %06.6d bytes read\n", 
	                    inPipe, i, ReadLen, nBytesRead);
				}
				// take care of perf stuff here
				else
				{
					totalBytes += nBytesRead;
				}
//                dump(pinBuf, nBytesRead); 
                
                if (WriteLen) {

                    //
                    // validate the input buffer against what
                    // we sent to the 82930 (loopback test)
                    //

                    ok = compare_buffs(pinBuf, poutBuf,  nBytesRead);

                    assert(ok == 1);

					if(ok != 1)
						fail++;

                    assert(ReadLen == WriteLen);
                    assert(nBytesRead == ReadLen);
                    assert(nBytesWrite == WriteLen);
                } else if (Iso){

                    //
                    // validate iso data
                    //
                }
	        }
	
        }

		// stop timing		
		if(PerfMode)
			finish = clock();

        if (pinBuf) {
            free(pinBuf);
        }

        if (poutBuf) {
            free(poutBuf);
        }

		// get perf data if enabled
		if(PerfMode)
		{
			I82930_PIPE_PERF_INFO	readPipe;
			I82930_PIPE_PERF_INFO	writePipe;

			if(hRead != INVALID_HANDLE_VALUE)
			{
				// turn off perf timing
				success = DeviceIoControl(hRead,
							IOCTL_I82930_STOP_PERF_TIMER,
							buf,
							sizeof(buf),
							buf,
							sizeof(buf),
							&nBytes,
							NULL);

				// get perf stats
				success = DeviceIoControl(hRead,
							IOCTL_I82930_RETURN_PERF_DATA,
							&readPipe,
							sizeof(readPipe),
							&readPipe,
							sizeof(readPipe),
							&nBytes,
							NULL);

				printf("<%s>   Driver Bytes/Second = %d   Cycles/Byte = %d\n", inPipe,
					   readPipe.BytesPerSecond, readPipe.ClockCyclesPerByte);
			}

			if(hWrite != INVALID_HANDLE_VALUE)
			{
				// turn off perf timing
				success = DeviceIoControl(hWrite,
							IOCTL_I82930_STOP_PERF_TIMER,
							buf,
							sizeof(buf),
							buf,
							sizeof(buf),
							&nBytes,
							NULL);

				// get perf stats
				success = DeviceIoControl(hWrite,
							IOCTL_I82930_RETURN_PERF_DATA,
							&writePipe,
							sizeof(writePipe),
							&writePipe,
							sizeof(writePipe),
							&nBytes,
							NULL);


				printf("<%s>   Driver Bytes/Second = %d   Cycles/Byte = %d\n", outPipe,
					   writePipe.BytesPerSecond, writePipe.ClockCyclesPerByte);
			}

			// get the number of seconds required to run test
			seconds = (double)(finish - start) / CLOCKS_PER_SEC;
			assert(seconds != 0.0);

			printf("App Bytes/Second All Pipes     = %d\n",
				   (ULONG) ( (double) totalBytes / seconds));
			
		}


		// close devices if needed
		if(hRead != INVALID_HANDLE_VALUE)
			CloseHandle(hRead);
		if(hWrite != INVALID_HANDLE_VALUE)
			CloseHandle(hWrite);

		if(TestMode)
		{
			if(fail)
				printf("Test failed\n");
			else
				printf("Test passed\n");
		}
    }		

	return 0;
}
