/*++

Copyright (c) 1993  Microsoft Corporation

Module Name:

    I82930.h

Abstract:



Environment:

    Kernel & user mode

Revision History:

    5-10-96 : created

--*/

#ifdef DRIVER

#define I82930_NAME_MAX  64

//
// we support up to 10 pipe handles
//
#define I82930_MAX_PIPES 10

//
// defines used for ISO test
//

#define I82930_MAX_IRP  2
#define I82930_NUM_ISO_PACKETS_PER_REQUEST  32
#define I82930_MAX_STREAM_PACKET 8

#define I82930_STREAM_TIMEOUT_INTERVAL  100


typedef struct _I82930_PIPE {
    BOOLEAN Opened;
    UCHAR Pad[3];
    PUSBD_PIPE_INFORMATION PipeInfo;
    WCHAR Name[I82930_NAME_MAX];
	LARGE_INTEGER BytesXfered;
	LARGE_INTEGER ElapsedTime;
	LARGE_INTEGER TimerStart;
	LARGE_INTEGER ElapsedCycles;
	LARGE_INTEGER CyclesStart;
	BOOLEAN		  bPerfTimerEnabled;
} I82930_PIPE, *PI82930_PIPE;

typedef struct _I82930_RW_CONTEXT {
    PURB Urb;
    PDEVICE_OBJECT DeviceObject;
} I82930_RW_CONTEXT, *PI82930_RW_CONTEXT;

#define MAX_INTERFACE 2

//
// A structure representing the instance information associated with
// this particular device.
//

typedef struct _DEVICE_EXTENSION {

    // Device object we call when submitting Urbs
    PDEVICE_OBJECT TopOfStackDeviceObject;

    PDEVICE_OBJECT PhysicalDeviceObject;

    DEVICE_POWER_STATE CurrentDevicePowerState;

    // configuration handle for the configuration the
    // device is currently in
    USBD_CONFIGURATION_HANDLE ConfigurationHandle;

    // ptr to the USB device descriptor
    // for this device
    PUSB_DEVICE_DESCRIPTOR DeviceDescriptor;

    // we support one interface
    // this is a copy of the info structure
    // returned from select_configuration or
    // select_interface
    PUSBD_INTERFACE_INFORMATION Interface;

    DEVICE_CAPABILITIES DeviceCapabilities;

    PVOID UserEvent;
    PIRP PowerIrp;
    KEVENT RemoveEvent;
    ULONG PendingIoCount;

    // Name buffer for our named Functional device object link
    WCHAR DeviceLinkNameBuffer[I82930_NAME_MAX];

    BOOLEAN AcceptingRequests;

    UCHAR Pad[3];

    I82930_PIPE PipeList[I82930_MAX_PIPES];

} DEVICE_EXTENSION, *PDEVICE_EXTENSION;

typedef struct _I82930_TRANSFER_OBJECT {
    struct _I82930_STREAM_OBJECT *StreamObject;
    PIRP Irp;
    PURB Urb;
    PUCHAR DataBuffer;
} I82930_TRANSFER_OBJECT, *PI82930_TRANSFER_OBJECT;

typedef struct _I82930_STREAM_OBJECT {
    PDEVICE_OBJECT DeviceObject;
    ULONG PendingIrps;
    PIRP StopIrp;
    PI82930_TRANSFER_OBJECT TransferObjectList[I82930_MAX_IRP];
    PUSBD_PIPE_INFORMATION PipeInfo;
    KDPC TimeoutDpc;
    KTIMER  TimeoutTimer;

    BOOLEAN EnableTimeoutDPC;
    BOOLEAN StreamError;
} I82930_STREAM_OBJECT, *PI82930_STREAM_OBJECT;

#if DBG

#define I82930_KdPrint(_x_) DbgPrint _x_ ;

#ifdef NTKERN
#define TRAP() _asm {int 3}
#else
#define TRAP() DbgBreakPoint()
#endif

#else

#define I82930_KdPrint(_x_)

#define TRAP()

#endif

NTSTATUS
I82930_Dispatch(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP           Irp
    );

VOID
I82930_Unload(
    IN PDRIVER_OBJECT DriverObject
    );

NTSTATUS
I82930_StartDevice(
    IN  PDEVICE_OBJECT DeviceObject
    );

NTSTATUS
I82930_StopDevice(
    IN  PDEVICE_OBJECT DeviceObject
    );

NTSTATUS
I82930_RemoveDevice(
    IN  PDEVICE_OBJECT DeviceObject
    );

NTSTATUS
I82930_CallUSBD(
    IN PDEVICE_OBJECT DeviceObject,
    IN PURB Urb
    );

NTSTATUS
I82930_PnPAddDevice(
    IN PDRIVER_OBJECT DriverObject,
    IN PDEVICE_OBJECT PhysicalDeviceObject
    );

NTSTATUS
I82930_CreateDeviceObject(
    IN PDRIVER_OBJECT DriverObject,
    IN PDEVICE_OBJECT *DeviceObject
    );

NTSTATUS
I82930_ConfigureDevice(
    IN  PDEVICE_OBJECT DeviceObject
    );

NTSTATUS
I82930_Write(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

NTSTATUS
I82930_Create(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

NTSTATUS
I82930_Read(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

NTSTATUS
I82930_ProcessIOCTL(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

NTSTATUS
I82930_SelectInterface(
    IN PDEVICE_OBJECT DeviceObject,
    IN PUSB_CONFIGURATION_DESCRIPTOR ConfigurationDescriptor,
    IN PUSBD_INTERFACE_INFORMATION Interface
    );

PUSB_CONFIGURATION_DESCRIPTOR
I82930_GetConfigDescriptor(
    IN PDEVICE_OBJECT DeviceObject
    );

NTSTATUS
I82930_BuildPipeList(
    IN  PDEVICE_OBJECT DeviceObject
    );

NTSTATUS
I82930_Close(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    );

NTSTATUS
I82930_ResetPipe(
    IN PDEVICE_OBJECT DeviceObject,
    IN PI82930_PIPE Pipe,
    IN BOOLEAN IsoClearStall
    );

NTSTATUS
I82930_StopIsoStream(
    IN PDEVICE_OBJECT DeviceObject,
    IN PI82930_STREAM_OBJECT StreamObject,
    IN PIRP Irp
    );    

NTSTATUS
I82930_StartIsoStream(
    IN PDEVICE_OBJECT DeviceObject,
    IN PUSBD_PIPE_INFORMATION PipeInfo,
    IN PIRP Irp
    );    

NTSTATUS
I82930_IsoIrp_Complete(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
	IN PVOID Context
    );   

VOID
I82930_IncrementIoCount(
    IN PDEVICE_OBJECT DeviceObject
    );

LONG
I82930_DecrementIoCount(
    IN PDEVICE_OBJECT DeviceObject
    );   

NTSTATUS
I82930_ReconfigureDevice(
    IN  PDEVICE_OBJECT DeviceObject
    );

NTSTATUS
I82930_ProcessPowerIrp(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP           Irp
    );    

#endif


