/*++

Copyright (c) 1995  Microsoft Corporation

Module Name:

   iso.c

Abstract:

   iso ping pong test

Environment:

    kernel mode only

Notes:

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
  PURPOSE.

  Copyright (c) 1996 Microsoft Corporation.  All Rights Reserved.


Revision History:

    5-4-96 : created

--*/

#define DRIVER

#include "wdm.h"
#include "stdarg.h"
#include "stdio.h"

#include "usbdi.h"
#include "usbdlib.h"
#include "I82930.h"


NTSTATUS
I82930_ProcessTransfer(
    IN PI82930_TRANSFER_OBJECT TransferObject
    )
/*++

Routine Description:

Arguments:

Return Value:

    NT status code

--*/
{
    NTSTATUS ntStatus;
    PIRP irp;

    irp = TransferObject->Irp;
    ntStatus = irp->IoStatus.Status;

    //
    // check the irp
    //

    if (!NT_SUCCESS(ntStatus)) {
        I82930_KdPrint (("'I82930.SYS: Iso IRP status = 0x%x\n", ntStatus));        

        TRAP();
    }

    //
    // now check the urb
    //
    
    return ntStatus;
}


NTSTATUS
I82930_InitializeStreamUrb(
    IN PDEVICE_OBJECT DeviceObject,
    IN PI82930_TRANSFER_OBJECT TransferObject
    )
/*++

Routine Description:

Arguments:

Return Value:

    NT status code

--*/
{
    NTSTATUS ntStatus = STATUS_SUCCESS;
    ULONG siz;
    ULONG packetSize, numPackets, i;
    PURB urb;
    PI82930_STREAM_OBJECT streamObject = TransferObject->StreamObject;

    urb = TransferObject->Urb;

    packetSize = streamObject->PipeInfo->MaximumPacketSize;
    numPackets = I82930_NUM_ISO_PACKETS_PER_REQUEST;
    
    siz = GET_ISO_URB_SIZE(numPackets);

    I82930_KdPrint (("'I82930.SYS: siz = 0x%x urb 0x%x\n", siz, urb));

    RtlZeroMemory(urb, siz);

    urb->UrbIsochronousTransfer.Hdr.Length = (USHORT) siz;
    urb->UrbIsochronousTransfer.Hdr.Function =
                URB_FUNCTION_ISOCH_TRANSFER;
    urb->UrbIsochronousTransfer.PipeHandle =
               streamObject->PipeInfo->PipeHandle;
    urb->UrbIsochronousTransfer.TransferFlags = USBD_TRANSFER_DIRECTION_IN;

    urb->UrbIsochronousTransfer.TransferBufferMDL = NULL;
    urb->UrbIsochronousTransfer.TransferBuffer = TransferObject->DataBuffer;
    
    urb->UrbIsochronousTransfer.TransferBufferLength =
        numPackets * packetSize;

    // start sending/receiving righ away
    urb->UrbIsochronousTransfer.TransferFlags |=
            USBD_START_ISO_TRANSFER_ASAP;

    urb->UrbIsochronousTransfer.NumberOfPackets = numPackets;
    urb->UrbIsochronousTransfer.UrbLink = 0;

    for (i=0; i< urb->UrbIsochronousTransfer.NumberOfPackets; i++) {
        urb->UrbIsochronousTransfer.IsoPacket[i].Offset
                    = i * packetSize;
    }

    I82930_KdPrint (("'I82930.SYS: Init Stream iso urb Length = 0x%x buf = 0x%x\n",
        urb->UrbIsochronousTransfer.TransferBufferLength,
        urb->UrbIsochronousTransfer.TransferBuffer));

    return ntStatus;
}


NTSTATUS
I82930_StartTransfer(
    IN PDEVICE_OBJECT DeviceObject,
    IN PI82930_STREAM_OBJECT StreamObject,
    IN ULONG Index
    )
/*++

Routine Description:

Arguments:

Return Value:

    nt status code

--*/
{
    NTSTATUS ntStatus = STATUS_SUCCESS;
    CCHAR stackSize;
    PDEVICE_EXTENSION deviceExtension;
    PI82930_TRANSFER_OBJECT transferObject;

    I82930_KdPrint (("'I82930.SYS: enter I82930_StartTransfer \n"));

    deviceExtension = DeviceObject->DeviceExtension;
    
    transferObject = ExAllocatePool(NonPagedPool, sizeof(I82930_TRANSFER_OBJECT));

    if (transferObject) {
        PIRP irp;

        I82930_KdPrint (("'I82930.SYS: TransferObject = %x \n", transferObject));

        transferObject->DataBuffer = NULL;
        transferObject->Urb = NULL;
    
        StreamObject->TransferObjectList[Index] = 
            transferObject;

        transferObject->StreamObject = StreamObject;

        stackSize = (CCHAR)(deviceExtension->TopOfStackDeviceObject->StackSize + 1);

        transferObject->Irp = 
            irp = IoAllocateIrp(stackSize,
                                FALSE);

        transferObject->DataBuffer = ExAllocatePool(NonPagedPool, 
                                                    I82930_NUM_ISO_PACKETS_PER_REQUEST * 
                                                    StreamObject->PipeInfo->MaximumPacketSize); 
                        
        transferObject->Urb = 
            ExAllocatePool(NonPagedPool, GET_ISO_URB_SIZE(I82930_NUM_ISO_PACKETS_PER_REQUEST));
         
        if (transferObject->Urb && transferObject->DataBuffer) {
        
            PIO_STACK_LOCATION nextStack; 
            NTSTATUS status;
            
            I82930_InitializeStreamUrb(DeviceObject, transferObject);

            nextStack = IoGetNextIrpStackLocation(irp);
            ASSERT(nextStack != NULL);

            nextStack->Parameters.Others.Argument1 = transferObject->Urb;
            nextStack->Parameters.DeviceIoControl.IoControlCode = 
                IOCTL_INTERNAL_USB_SUBMIT_URB;                    
            nextStack->MajorFunction = IRP_MJ_INTERNAL_DEVICE_CONTROL;

            IoSetCompletionRoutine(irp,
                    I82930_IsoIrp_Complete,
                    transferObject,
                    TRUE,
                    TRUE,
                    TRUE);

            //
            // submit the request
            //

            StreamObject->PendingIrps++;
            status = IoCallDriver(deviceExtension->TopOfStackDeviceObject, 
                                  irp);

        } else {
            ntStatus = STATUS_INSUFFICIENT_RESOURCES;

            if (transferObject->DataBuffer) { 
                ExFreePool(transferObject->DataBuffer);
            }

            if (transferObject->Urb) { 
                ExFreePool(transferObject->Urb);
            }
            
        }
            
    } else {
        ntStatus = STATUS_INSUFFICIENT_RESOURCES;
    }

    I82930_KdPrint (("'I82930.SYS: exit I82930_StartTransfer %x\n", ntStatus));
    
    return ntStatus;
}


NTSTATUS
I82930_IsoIrp_Complete(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
	IN PVOID Context
    )
/*++

Routine Description:

    This routine is called when the port driver completes an IRP.  

Arguments:

    DeviceObject - Pointer to the device object for the class device.

    Irp - Irp completed.

	Context - Driver defined context, points to a transfer extension structure 
    	for a pair of parallel iso requests.

Return Value:

    The function value is the final status from the operation.

--*/
{
    NTSTATUS status;
    PDEVICE_EXTENSION deviceExtension;
    PDEVICE_OBJECT deviceObject;
    PI82930_TRANSFER_OBJECT transferObject;
    PI82930_STREAM_OBJECT streamObject;
    PIO_STACK_LOCATION nextStack;
    

    transferObject = Context;
    streamObject = transferObject->StreamObject;    
    deviceObject = streamObject->DeviceObject;
    deviceExtension = deviceObject->DeviceExtension;

    streamObject->PendingIrps--;

    status = I82930_ProcessTransfer(transferObject);

    if (NT_SUCCESS(status)) {
        I82930_InitializeStreamUrb(DeviceObject, transferObject);

        nextStack = IoGetNextIrpStackLocation(Irp);
        ASSERT(nextStack != NULL);

        nextStack->Parameters.Others.Argument1 = transferObject->Urb;
        nextStack->Parameters.DeviceIoControl.IoControlCode = 
            IOCTL_INTERNAL_USB_SUBMIT_URB;                    
        nextStack->MajorFunction = IRP_MJ_INTERNAL_DEVICE_CONTROL;

        IoSetCompletionRoutine(Irp,
                I82930_IsoIrp_Complete,
                transferObject,
                TRUE,
                TRUE,
                TRUE);

        //
        // submit the request
        //

        streamObject->PendingIrps++;
        status = IoCallDriver(deviceExtension->TopOfStackDeviceObject, 
                              Irp);
    }
    
    return STATUS_MORE_PROCESSING_REQUIRED;
}               


VOID
I82930_StreamTimeoutDPC(
    IN PKDPC Dpc,
    IN PVOID DeferredContext,
    IN PVOID SystemArgument1,
    IN PVOID SystemArgument2
    )
/*++

Routine Description:

Arguments:

Return Value:

    The function value is the final status from the operation.

--*/
{
    PDEVICE_EXTENSION deviceExtension;
    PDEVICE_OBJECT deviceObject;
    PI82930_STREAM_OBJECT streamObject;
    BOOLEAN inQueue;
    LARGE_INTEGER dueTime;

    streamObject = DeferredContext;    
    deviceObject = streamObject->DeviceObject;
    deviceExtension = deviceObject->DeviceExtension;

    if (streamObject->PendingIrps == 0 && 
        streamObject->StopIrp) {
    
        //
        // no more irps are pending and we have a stop
        // irp then signal it.
        //

        streamObject->StopIrp = NULL;

        
    }

    //
    // schedule next one
    //
    
    if (streamObject->EnableTimeoutDPC) {

        KeInitializeTimer(&streamObject->TimeoutTimer);
        KeInitializeDpc(&streamObject->TimeoutDpc,
                        I82930_StreamTimeoutDPC,
                        streamObject);

        dueTime.QuadPart = -10000 * I82930_STREAM_TIMEOUT_INTERVAL;

        inQueue = KeSetTimer(&streamObject->TimeoutTimer,
                             dueTime,
                             &streamObject->TimeoutDpc);        

        ASSERT(inQueue == FALSE);                             
        
    }
}


NTSTATUS
I82930_StartIsoStream(
    IN PDEVICE_OBJECT DeviceObject,
    IN PUSBD_PIPE_INFORMATION PipeInfo,
    IN PIRP Irp
    )
/*++

Routine Description:

Arguments:

    DeviceObject - pointer to the device extension for this instance of the
                     82930 device.

    Irp -

Return Value:

    NT status code

--*/
{
    ULONG i;
    NTSTATUS ntStatus = STATUS_SUCCESS;
    PIO_STACK_LOCATION irpStack;
    ULONG inputBufferLength;
    ULONG outputBufferLength;
    PULONG streamObjectHandle;
    PI82930_STREAM_OBJECT streamObject;
    LARGE_INTEGER dueTime;
    BOOLEAN inQueue;

    I82930_KdPrint (("'I82930.SYS: enter I82930_StartIsoStream  pipe %x\n", PipeInfo));

    irpStack = IoGetCurrentIrpStackLocation (Irp);
     
    streamObjectHandle = Irp->AssociatedIrp.SystemBuffer;
    inputBufferLength  = irpStack->Parameters.DeviceIoControl.InputBufferLength;
    outputBufferLength = irpStack->Parameters.DeviceIoControl.OutputBufferLength;

    streamObject = ExAllocatePool(NonPagedPool, sizeof(I82930_STREAM_OBJECT));

    if (streamObject) {

        streamObject->StopIrp = NULL;
        streamObject->DeviceObject = DeviceObject;
        streamObject->PendingIrps = 0;
        streamObject->StreamError = FALSE;
        streamObject->PipeInfo = PipeInfo;

        for (i=0; i<I82930_MAX_IRP; i++) {
            streamObject->TransferObjectList[i] = NULL;
        }
        
        for (i=0; i<I82930_MAX_IRP; i++) {

            ntStatus = I82930_StartTransfer(DeviceObject,
                                            streamObject,
                                            i);

            if (!NT_SUCCESS(ntStatus)) {
                TRAP();
                break;
            }
            
        }

        *streamObjectHandle = (ULONG) streamObject;

        //
        // start the timeout DPC
        //
        streamObject->EnableTimeoutDPC = TRUE;

        KeInitializeTimer(&streamObject->TimeoutTimer);
        KeInitializeDpc(&streamObject->TimeoutDpc,
                        I82930_StreamTimeoutDPC,
                        streamObject);

        dueTime.QuadPart = -10000 * I82930_STREAM_TIMEOUT_INTERVAL;

        inQueue = KeSetTimer(&streamObject->TimeoutTimer,
                             dueTime,
                             &streamObject->TimeoutDpc);        

        
    } else {
        ntStatus = STATUS_INSUFFICIENT_RESOURCES;
    }

    Irp->IoStatus.Information = sizeof(*streamObjectHandle);                
    Irp->IoStatus.Status = ntStatus;

    I82930_KdPrint (("'I82930.SYS: exit I82930_StartIsoStream %x\n", ntStatus));

    return ntStatus;
}


NTSTATUS
I82930_StopIsoStream(
    IN PDEVICE_OBJECT DeviceObject,
    IN PI82930_STREAM_OBJECT StreamObject,
    IN PIRP Irp
    )
/*++

Routine Description:

Arguments:

    DeviceObject - pointer to the device extension for this instance of the
                     82930 device.

    Irp -

Return Value:

    NT status code

--*/
{

    TRAP();
    
}



