//==========================================================================;
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE.
//
//  Copyright (c) 1992 - 1996  Microsoft Corporation.  All Rights Reserved.
//
//==========================================================================;

#ifndef __CAPMAIN_H__
#define __CAPMAIN_H__

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

// Mediums define hardwired connections between filters

#define STATIC_MEDIUM_VIDEO_TESTCAP_ANALOGVIDEOIN \
    0xfb6c4240, 0x353, 0x11d1, 0x90, 0x5f, 0x0, 0x0, 0xc0, 0xcc, 0x16, 0xba
DEFINE_GUIDSTRUCT("FB6C4240-0353-11d1-905F-0000C0CC16BA", MEDIUM_VIDEO_TESTCAP_ANALOGVIDEOIN);
#define MEDIUM_VIDEO_TESTCAP_ANALOGVIDEOIN DEFINE_GUIDNAMED(MEDIUM_VIDEO_TESTCAP_ANALOGVIDEOIN)


/*****************************************************************************
*
* The following structures are samples of information that could be used in
* a device extension structure
*
*****************************************************************************/
typedef struct *PSTREAMX;

//
// definition of the full HW device extension structure This is the structure
// that will be allocated in HW_INITIALIZATION by the stream class driver
// Any information that is used in processing a device request (as opposed to
// a STREAM based request) should be in this structure.  A pointer to this
// structure will be passed in all requests to the minidriver. (See
// HW_STREAM_REQUEST_BLOCK in STRMINI.H)
//

typedef struct _HW_DEVICE_EXTENSION {
    PULONG                   ioBaseLocal;           // board base address
    USHORT                   Irq;                   // irq level
    BOOLEAN                  IRQExpected;           // IRQ expected
    PSTREAMX                 pStrmEx [4];           // Pointers to each stream
    UINT                     ActualInstances [4];   // Counter of instances per stream

    // Crossbar settings
    LONG                     InputConnected;        // 0 to nInputPins

    // TV Tuner settings
    ULONG                    Frequency;
    ULONG                    VideoStandard;
    ULONG                    TuningQuality;
    ULONG                    TunerInput;
    ULONG                    Busy;

    // TV Audio settings
    ULONG                    TVAudioMode;

    // VideoProcAmp settings
    LONG                     Brightness;
    LONG                     Contrast;
    
    // Clock 
    REFERENCE_TIME           QST_Start;             // KeQuerySystemTime at run
    REFERENCE_TIME           QST_Now;               // KeQuerySystemTime currently
    REFERENCE_TIME           WallTime100ns;         // elapsed time based on KeQueryPerformanceCounter

    // Channel Change information
    KS_TVTUNER_CHANGE_INFO   TVTunerChangeInfo;

} HW_DEVICE_EXTENSION, *PHW_DEVICE_EXTENSION;

//
// this structure is our per stream extension structure.  This stores
// information that is relevant on a per stream basis.  Whenever a new stream
// is opened, the stream class driver will allocate whatever extension size
// is specified in the HwInitData.PerStreamExtensionSize.
//
 
typedef struct _STREAMEX {
    PHW_DEVICE_EXTENSION        pHwDevExt;          // For timer use
    PHW_STREAM_OBJECT           pStreamObject;      // For timer use
    KS_VIDEOINFOHEADER         *pVideoInfoHeader;   // format (variable size!)
    KS_FRAME_INFO               FrameInfo;          // PictureNumber, etc.
    ULONG                       fDiscontinuity;     // Discontinuity since last valid
    KSSTATE                     KSState;            // Run, Stop, Pause
    REFERENCE_TIME              FrameTime100ns;     // elapsed time based on frames captured
    PHW_STREAM_REQUEST_BLOCK    pCurrentSrb;        // video request in progress
    PVOID                       pDMABuf;            // pointer to the video DMA buffer
    STREAM_PHYSICAL_ADDRESS     pPhysDmaBuf;        // physical address of DMA buffer
    ULONG                       cDmaBuf;            // size of DMA buffer
    HANDLE                      hMasterClock;       // 
    HANDLE                      hClock;             //
    UCHAR                       LineBuffer[720 * 3];// working buffer (RGB24)
} STREAMEX, *PSTREAMEX;

//
// this structure defines the per request extension.  It defines any storage
// space that the min driver may need in each request packet.
//

typedef struct _SRB_EXTENSION {

    ULONG foo;

} SRB_EXTENSION, * PSRB_EXTENSION;

/*****************************************************************************
*
* the following section defines prototypes for the minidriver initialization
* routines
*
******************************************************************************/

//
// DriverEntry:
//
// This routine is called when the mini driver is first loaded.  The driver
// should then call the StreamClassRegisterAdapter function to register with
// the stream class driver
//

ULONG DriverEntry (PVOID Context1, PVOID Context2);

//
// This routine is called by the stream class driver with configuration
// information for an adapter that the mini driver should load on.  The mini
// driver should still perform a small verification to determine that the
// adapter is present at the specified addresses, but should not attempt to
// find an adapter as it would have with previous NT miniports.
//
// All initialization of the adapter should also be performed at this time.
//

BOOLEAN HwInitialize (IN OUT PHW_STREAM_REQUEST_BLOCK pSrb);

//
// This routine is called when the system is going to remove or disable the
// device.
//
// The mini-driver should free any system resources that it allocated at this
// time.  Note that system resources allocated for the mini-driver by the
// stream class driver will be free'd by the stream driver, and should not be
// free'd in this routine.  (Such as the HW_DEVICE_EXTENSION)
//

BOOLEAN HwUnInitialize ( PHW_STREAM_REQUEST_BLOCK pSrb);


BOOLEAN HwQueryUnload ( PHW_STREAM_REQUEST_BLOCK pSrb);


//
// This is the prototype for the Hardware Interrupt Handler.  This routine
// will be called whenever the minidriver receives an interrupt
//

BOOLEAN HwInterrupt ( IN PHW_DEVICE_EXTENSION pDeviceExtension );

//
// This is the prototype for the stream enumeration function.  This routine
// provides the stream class driver with the information on data stream types
// supported
//

VOID AdapterStreamInfo(PHW_STREAM_REQUEST_BLOCK pSrb);

//
// This is the prototype for the stream open function
//

VOID AdapterOpenStream(PHW_STREAM_REQUEST_BLOCK pSrb);

//
// This is the prototype for the stream close function
//

VOID AdapterCloseStream(PHW_STREAM_REQUEST_BLOCK pSrb);

//
// This is the prototype for the AdapterReceivePacket routine.  This is the
// entry point for command packets that are sent to the adapter (not to a
// specific open stream)
//

VOID STREAMAPI AdapterReceivePacket(IN PHW_STREAM_REQUEST_BLOCK Srb);

//
// This is the protoype for the cancel packet routine.  This routine enables
// the stream class driver to cancel an outstanding packet.
//

VOID STREAMAPI AdapterCancelPacket(IN PHW_STREAM_REQUEST_BLOCK Srb);

//
// This is the packet timeout function.  The adapter may choose to ignore a
// packet timeout, or rest the adapter and cancel the requests, as required.
//

VOID STREAMAPI AdapterTimeoutPacket(IN PHW_STREAM_REQUEST_BLOCK Srb);

VOID STREAMAPI AdapterGetProperty(IN PHW_STREAM_REQUEST_BLOCK Srb);
VOID STREAMAPI AdapterSetProperty(IN PHW_STREAM_REQUEST_BLOCK Srb);

BOOL 
AdapterVerifyFormat(
        PKSDATAFORMAT pKSDataFormatToVerify, 
        int StreamNumber);

BOOL
AdapterFormatFromRange(
        IN PHW_STREAM_REQUEST_BLOCK pSrb);

void 
CompleteStreamSRB (
         IN PHW_STREAM_REQUEST_BLOCK pSrb, 
         STREAM_MINIDRIVER_STREAM_NOTIFICATION_TYPE NotificationType1,
         BOOL fUseNotification2,
         STREAM_MINIDRIVER_STREAM_NOTIFICATION_TYPE NotificationType2
        );
void 
CompleteDeviceSRB (
         IN PHW_STREAM_REQUEST_BLOCK pSrb, 
         IN STREAM_MINIDRIVER_DEVICE_NOTIFICATION_TYPE NotificationType,
         BOOL fReadyForNext
        );

//
// prototypes for data handling routines
//
void CompleteStreamIRP (IN PHW_STREAM_REQUEST_BLOCK pSrb, BOOLEAN ReadyForNext);

VOID STREAMAPI VideoReceiveDataPacket(IN PHW_STREAM_REQUEST_BLOCK pSrb);
VOID STREAMAPI VideoReceiveCtrlPacket(IN PHW_STREAM_REQUEST_BLOCK pSrb);
void EnableIRQ(PHW_STREAM_OBJECT pstrm);
void DisableIRQ(PHW_STREAM_OBJECT pstrm);

//
// prototypes for properties and states
//

VOID VideoSetState(PHW_STREAM_REQUEST_BLOCK pSrb);
VOID VideoGetState(PHW_STREAM_REQUEST_BLOCK pSrb);
VOID VideoGetProperty(PHW_STREAM_REQUEST_BLOCK pSrb);
VOID VideoStreamGetConnectionProperty (PHW_STREAM_REQUEST_BLOCK pSrb);

// 
// stream clock functions
//
ULONGLONG 
VideoGetSystemTime(
    );
VOID STREAMAPI StreamClockRtn(IN PHW_TIME_CONTEXT TimeContext);
VOID VideoOpenCloseMasterClock (PHW_STREAM_REQUEST_BLOCK pSrb);
VOID VideoIndicateMasterClock (PHW_STREAM_REQUEST_BLOCK pSrb);

#ifdef    __cplusplus
}
#endif // __cplusplus

#endif //__CAPMAIN_H__

