//==========================================================================;
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE.
//
//  Copyright (c) 1992 - 1997  Microsoft Corporation.  All Rights Reserved.
//
//==========================================================================;

#include "strmini.h"
#include "ksmedia.h"
#include "capmain.h"
#include "capdebug.h"
#include "capxfer.h"

//
// EIA-189-A Standard color bar definitions
//

// 75% Amplitude, 100% Saturation
const static UCHAR NTSCColorBars75Amp100SatRGB24 [3][8] = 
{
//  Whi Yel Cya Grn Mag Red Blu Blk
    191,191,  0,  0,191,191,  0,  0,    // Red
    191,191,191,191,  0,  0,  0,  0,    // Green
    191,  0,191,  0,191,  0,191,  0     // Blue
};

// 100% Amplitude, 100% Saturation
const static UCHAR NTSCColorBars100Amp100SatRGB24 [3][8] = 
{
//  Whi Yel Cya Grn Mag Red Blu Blk
    255,255,  0,  0,255,255,  0,  0,    // Red
    255,255,255,255,  0,  0,  0,  0,    // Green
    255,  0,255,  0,255,  0,255,  0     // Blue
};


/*
** ImageSynth()
**
**   Synthesizes NTSC color bars, white, black, and grayscale images
**
** Arguments:
**
**   pSrb - The stream request block for the Video stream
**   ImageXferCommands - Index specifying the image to generate
**
** Returns:
**
**   Nothing
**
** Side Effects:  none
*/

void ImageSynth (
    IN OUT PHW_STREAM_REQUEST_BLOCK pSrb,
    IN ImageXferCommands Command
    )
{
    PSTREAMEX               pStrmEx = (PSTREAMEX)pSrb->StreamObject->HwStreamExtension;
    PHW_DEVICE_EXTENSION    pHwDevExt = ((PHW_DEVICE_EXTENSION)pSrb->HwDeviceExtension);
    int                     StreamNumber = pSrb->StreamObject->StreamNumber;
    KS_VIDEOINFOHEADER      *pVideoInfoHdr = pStrmEx->pVideoInfoHeader;

    UINT biWidth        =   pVideoInfoHdr->bmiHeader.biWidth;
    UINT biHeight       =   pVideoInfoHdr->bmiHeader.biHeight;
    UINT biSizeImage    =   pVideoInfoHdr->bmiHeader.biSizeImage;
    UINT biWidthBytes   =   KS_DIBWIDTHBYTES (pVideoInfoHdr->bmiHeader);
    UINT biBitCount     =   pVideoInfoHdr->bmiHeader.biBitCount;

    UINT                    Line;
    PUCHAR                  pLineBuffer;

    PKSSTREAM_HEADER        pDataPacket = pSrb->CommandData.DataBufferArray;
    PUCHAR                  pImage =  pDataPacket->Data;

    DEBUG_ASSERT (pSrb->NumberOfBuffers == 1);

#if 0
    // Note:  set "ulInDebug = 1" in a debugger to view this output with .ntkern
    KdPrint(("\'TestCap: ImageSynthBegin\n"));
    KdPrint(("\'TestCap: biSizeImage=%d, DataPacketLength=%d\n", 
            biSizeImage, pDataPacket->DataPacketLength));
    KdPrint(("\'TestCap: biWidth=%d biHeight=%d WidthBytes=%d bpp=%d\n", 
            biWidth, biHeight, biWidthBytes, biBitCount));
    KdPrint(("\'TestCap: pImage=%x\n", pImage));
#endif

    // 
    // Synthesize a single line of image data, which will then be replicated
    //

    pLineBuffer = &pStrmEx->LineBuffer[0];

    switch (biBitCount) {

    case 24: 
    {

        switch (Command) {

        case IMAGE_XFER_NTSC_EIA_100AMP_100SAT:
            // 100% saturation
            {
                UINT x, col;
                PUCHAR pT = pLineBuffer;
        
                for (x = 0; x < biWidth; x++) {
                    col = (x * 8) / biWidth;
                    *pT++ = NTSCColorBars100Amp100SatRGB24[0][col]; // Red
                    *pT++ = NTSCColorBars100Amp100SatRGB24[1][col]; // Green
                    *pT++ = NTSCColorBars100Amp100SatRGB24[2][col]; // Blue
                }
            }
            break;

        case IMAGE_XFER_NTSC_EIA_75AMP_100SAT:
            // 75% Saturation
            {
                UINT x, col;
                PUCHAR pT = pLineBuffer;
        
                for (x = 0; x < biWidth; x++) {
                    col = (x * 8) / biWidth;
                    *pT++ = NTSCColorBars75Amp100SatRGB24[0][col]; // Red
                    *pT++ = NTSCColorBars75Amp100SatRGB24[1][col]; // Green
                    *pT++ = NTSCColorBars75Amp100SatRGB24[2][col]; // Blue
                }
            }
            break;

        case IMAGE_XFER_BLACK:
            // All black
            RtlFillMemory(
		        pLineBuffer,
		        biWidthBytes,
		        (UCHAR) 0);
            break;

        case IMAGE_XFER_WHITE:
            // All white
            RtlFillMemory(
		        pLineBuffer,
		        biWidthBytes,
		        (UCHAR) 255);
            break;

        case IMAGE_XFER_GRAY_INCREASING:
            // grayscale ramp
            RtlFillMemory(
		        pLineBuffer,
		        biWidthBytes,
		        (UCHAR) (pStrmEx->FrameInfo.PictureNumber * 8));
            break;

        default:
            break;
        }

    } // switch 24

    break;

    case 8: 
    {

        switch (Command) {

        case IMAGE_XFER_NTSC_EIA_75AMP_100SAT:
            {
                // TODO
            }
            break;

        case IMAGE_XFER_NTSC_EIA_100AMP_100SAT:
            {
                // TODO
            }
            break;

        case IMAGE_XFER_BLACK:
            RtlFillMemory(
		        pLineBuffer,
		        biWidthBytes,
		        (UCHAR) 0);
            break;

        case IMAGE_XFER_WHITE:
            RtlFillMemory(
		        pLineBuffer,
		        biWidthBytes,
		        (UCHAR) 15);
            break;

        case IMAGE_XFER_GRAY_INCREASING:
            RtlFillMemory(
		        pLineBuffer,
		        biWidthBytes,
		        (UCHAR) (pStrmEx->FrameInfo.PictureNumber % 16));
            break;

        default:
            break;
        }

    } // switch 8

        break;

    default:
        break;
    }

    // 
    // Copy the single line synthesized to all rows of the image
    //

    for (Line = 0; Line < biHeight; Line++, pImage += biWidthBytes) {
        RtlCopyMemory(
                pImage,
		pLineBuffer,
		biWidthBytes);
    }

    //
    // Report back the actual number of bytes copied to the destination buffer
    // (This can be smaller than the allocated buffer for compressed images)
    //

    pDataPacket->DataUsed = biSizeImage;
}
