/*++

Copyright (c) 1996, 1997  Intel Corporation

Module Name:

   prpobj.c

Abstract:

   Code related to about properties 

Author:

    jdunn
    
Environment:

   Kernel mode only


Revision History:

--*/

#include "warn.h"
#include "wdm.h"

#include <strmini.h>
#include <ksmedia.h>

#include "usbdi.h"
#include "usbcamdi.h"

#include "intelcam.h"

#include "prpobj.h"

//
// First define all of the ranges and stepping values
//

// ------------------------------------------------------------------------
KSPROPERTY_STEPPING_LONG BrightnessRangeAndStep [] = 
{
    {
        STEPPING_DELTA_BRIGHTNESS,   // SteppingDelta
        0,                           // Reserved
        0,                           // Minimum in (IRE * 100) units
        MAX_BRIGHTNESS_IRE_UNITS     // Maximum in (IRE * 100) units
    }
};

KSPROPERTY_MEMBERSLIST BrightnessMembersList [] = 
{
    {
        {
            KSPROPERTY_MEMBER_RANGES,
            sizeof (BrightnessRangeAndStep),
            SIZEOF_ARRAY (BrightnessRangeAndStep),
            0
        },
        (PVOID) BrightnessRangeAndStep
    }    
};

KSPROPERTY_VALUES BrightnessValues =
{
    {
        STATICGUIDOF (KSPROPTYPESETID_General),
        VT_I4,
        0
    },
    SIZEOF_ARRAY (BrightnessMembersList),
    BrightnessMembersList
};



// ------------------------------------------------------------------------
DEFINE_KSPROPERTY_TABLE(VideoProcAmpProperties)
{
    DEFINE_KSPROPERTY_ITEM
    (
        KSPROPERTY_VIDEOPROCAMP_BRIGHTNESS,
        TRUE,                                   // GetSupported or Handler
        sizeof(KSPROPERTY_VIDEOPROCAMP_S),      // MinProperty
        sizeof(KSPROPERTY_VIDEOPROCAMP_S),      // MinData
        TRUE,                                   // SetSupported or Handler
        &BrightnessValues,                      // Values
        0,                                      // RelationsCount
        NULL,                                   // Relations
        NULL,                                   // SupportHandler
        sizeof(ULONG)                           // SerializedSize
    ),
    
};



//
// Property set for the Video Camera
//

DEFINE_KSPROPERTY_TABLE(CameraProperties)
{
    DEFINE_KSPROPERTY_ITEM
    (
        KSPROPERTY_CAMERACONTROL_PAN,           // PropertyId
        TRUE,                                   // GetSupported or Handler
        sizeof(ULONG),                          // MinProperty
        sizeof(ULONG),                          // MinData
        FALSE,                                  // SetSupported or Handler
        NULL,                                   // Values
        0,                                      // RelationsCount
        NULL,                                   // Relations
        NULL,                                   // SupportHandler
        sizeof(ULONG)                           // SerializedSize
    ),
    DEFINE_KSPROPERTY_ITEM
    (
        KSPROPERTY_CAMERACONTROL_TILT,
        TRUE,                                   // GetSupported or Handler
        sizeof(ULONG),                          // MinProperty
        sizeof(ULONG),                          // MinData
        FALSE,                                  // SetSupported or Handler
        NULL,                                   // Values
        0,                                      // RelationsCount
        NULL,                                   // Relations
        NULL,                                   // SupportHandler
        sizeof(ULONG)                           // SerializedSize
    ),
    DEFINE_KSPROPERTY_ITEM
    (
        KSPROPERTY_CAMERACONTROL_ROLL,
        TRUE,                                   // GetSupported or Handler
        sizeof(ULONG),                          // MinProperty
        sizeof(ULONG),                          // MinData
        FALSE,                                  // SetSupported or Handler
        NULL,                                   // Values
        0,                                      // RelationsCount
        NULL,                                   // Relations
        NULL,                                   // SupportHandler
        sizeof(ULONG)                           // SerializedSize
    ),
    DEFINE_KSPROPERTY_ITEM
    (
        KSPROPERTY_CAMERACONTROL_ZOOM,
        TRUE,                                   // GetSupported or Handler
        sizeof(ULONG),                          // MinProperty
        sizeof(ULONG),                          // MinData
        FALSE,                                  // SetSupported or Handler
        NULL,                                   // Values
        0,                                      // RelationsCount
        NULL,                                   // Relations
        NULL,                                   // SupportHandler
        sizeof(ULONG)                           // SerializedSize
    ),
    DEFINE_KSPROPERTY_ITEM
    (
        KSPROPERTY_CAMERACONTROL_EXPOSURE,
        TRUE,                                   // GetSupported or Handler
        sizeof(ULONG),                          // MinProperty
        sizeof(ULONG),                          // MinData
        FALSE,                                  // SetSupported or Handler
        NULL,                                   // Values
        0,                                      // RelationsCount
        NULL,                                   // Relations
        NULL,                                   // SupportHandler
        sizeof(ULONG)                           // SerializedSize
    ),
    DEFINE_KSPROPERTY_ITEM
    (
        KSPROPERTY_CAMERACONTROL_IRIS,
        TRUE,                                   // GetSupported or Handler
        sizeof(ULONG),                          // MinProperty
        sizeof(ULONG),                          // MinData
        FALSE,                                  // SetSupported or Handler
        NULL,                                   // Values
        0,                                      // RelationsCount
        NULL,                                   // Relations
        NULL,                                   // SupportHandler
        sizeof(ULONG)                           // SerializedSize
    ),
    DEFINE_KSPROPERTY_ITEM
    (
        KSPROPERTY_CAMERACONTROL_FOCUS,
        TRUE,                                   // GetSupported or Handler
        sizeof(ULONG),                          // MinProperty
        sizeof(ULONG),                          // MinData
        FALSE,                                  // SetSupported or Handler
        NULL,                                   // Values
        0,                                      // RelationsCount
        NULL,                                   // Relations
        NULL,                                   // SupportHandler
        sizeof(ULONG)                           // SerializedSize
    )
};


GUID PROPSETID_INTEL_CUSTOM_PROP =
    {0xCD8C22E0, 0xCB4C, 0x11CF, {0x85, 0xC8, 0xDC, 0x17, 0x88, 0x39, 0xEC, 0x74}};



DEFINE_KSPROPERTY_TABLE(CustomProperties)
{
    DEFINE_KSPROPERTY_ITEM
    (
        KSPROPERTY_CUSTOM_PROP_FIRMWARE_VER,    // PropertyId
        TRUE,                                   // GetSupported or Handler
        sizeof(KSPROPERTY_CUSTOM_PROP_S),       // MinProperty
        sizeof(KSPROPERTY_CUSTOM_PROP_S),       // MinData
        FALSE,                                  // SetSupported or Handler
        NULL,                                   // Values
        0,                                      // RelationsCount
        NULL,                                   // Relations
        NULL,                                   // SupportHandler
        sizeof(ULONG)                           // SerializedSize
    )
};
    

//
// All of the property sets supported by the adapter
//

DEFINE_KSPROPERTY_SET_TABLE(AdapterPropertyTable)
{
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_VIDCAP_VIDEOPROCAMP,                 // Set
        SIZEOF_ARRAY(VideoProcAmpProperties),           // PropertiesCount
        VideoProcAmpProperties,                         // PropertyItem
        0,                                              // FastIoCount
        NULL                                            // FastIoTable
    ),

    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_INTEL_CUSTOM_PROP,             // Set
        SIZEOF_ARRAY(CustomProperties),           // PropertiesCount
        CustomProperties,                         // PropertyItem
        0,                                        // FastIoCount
        NULL                                      // FastIoTable
    )

};


#define NUMBER_OF_ADAPTER_PROPERTY_SETS (SIZEOF_ARRAY (AdapterPropertyTable))

PVOID 
INTELCAM_GetAdapterPropertyTable(
    PULONG NumberOfPropertySets
    )
{
    *NumberOfPropertySets = NUMBER_OF_ADAPTER_PROPERTY_SETS;
    return (PVOID) AdapterPropertyTable;
}

////////////////
/////
/////  The following are Camera dependent
/////
////////////////

//
// Brightness.
//
const RANGE _Brightness =
{
    0, 9, 0, 9, 1,
    4                  // Default
};



/*
** INTELCAM_GetCameraProperty()
**
** Arguments:
**
**  DeviceContext - driver context
**
** Returns:
**
**  NT status completion code 
**  
** Side Effects:  none
*/
NTSTATUS
INTELCAM_GetCameraProperty(
    PINTELCAM_DEVICE_CONTEXT DeviceContext,
    PHW_STREAM_REQUEST_BLOCK Srb
    )
{
    PSTREAM_PROPERTY_DESCRIPTOR streamPropertyDescr = 
	    Srb->CommandData.PropertyInfo;
    PKSPROPERTY_VIDEOPROCAMP_S propertyData = 
        streamPropertyDescr->PropertyInfo;
    ULONG flags = propertyData->Flags;            
    // index of the property	    
    ULONG propertyID = streamPropertyDescr->Property->Id;     
    ULONG propValue, length;
    USHORT value, index;
    NTSTATUS status = STATUS_SUCCESS;
    
    ASSERT(streamPropertyDescr->PropertyOutputSize 
        >= sizeof(KSPROPERTY_VIDEOPROCAMP_S));

    switch(propertyID) {
    case  KSPROPERTY_VIDEOPROCAMP_BRIGHTNESS:
        length = sizeof(WORD);
        index = INDEX_PREF_BRIGHTNESS;

        if (flags == DEFAULT_VALUE) {
            value = (_Brightness.Default * STEPPING_DELTA_BRIGHTNESS);
        } else {
            status = USBCAMD_ControlVendorCommand(DeviceContext,
                                          REQUEST_GET_PREFERENCE,
                                          0,
                                          index,
                                          &value,
                                          &length,
                                          TRUE,
                                          NULL,
                                          NULL);
        }                                          
        
        if (NT_SUCCESS(status)) {
            propValue = (value * STEPPING_DELTA_BRIGHTNESS);

            Srb->ActualBytesTransferred = sizeof(KSPROPERTY_VIDEOPROCAMP_S);
			propertyData->Flags = KSPROPERTY_VIDEOPROCAMP_FLAGS_AUTO;

  			propertyData->Value = propValue;
        }                
        break;
        
    default:
        status = STATUS_NOT_SUPPORTED;
    }

    INTELCAM_KdPrint(MIN_TRACE, ("'Getting property <0x%x>\n", status));

    Srb->Status = status;    

    return status;
}


/*
** INTELCAM_SetCameraProperty()
**
** Arguments:
**
**  DeviceContext - driver context
**
** Returns:
**
**  NT status completion code 
**  
** Side Effects:  none
*/
NTSTATUS
INTELCAM_SetCameraProperty(
    PINTELCAM_DEVICE_CONTEXT DeviceContext,
    PHW_STREAM_REQUEST_BLOCK Srb
    )
{
	PSTREAM_PROPERTY_DESCRIPTOR streamPropertyDescr = 
	    Srb->CommandData.PropertyInfo;
    PKSPROPERTY_VIDEOPROCAMP_S propertyData = 
        streamPropertyDescr->PropertyInfo;
    ULONG flags = propertyData->Flags;            
    // index of the property	    
    ULONG propertyID = streamPropertyDescr->Property->Id;     
    ULONG value;
    USHORT regAddr;
    ULONG length;
    NTSTATUS status = STATUS_SUCCESS;

    value =  propertyData->Value;
    
    //
    // now set the camera
    //
    
    switch (propertyID) {
    case  KSPROPERTY_VIDEOPROCAMP_BRIGHTNESS:

        if (flags == DEFAULT_VALUE) {
            INTELCAM_KdPrint(MIN_TRACE, ("'Setting default property\n"));
            value = (_Brightness.Default * STEPPING_DELTA_BRIGHTNESS);
        } 

        length = sizeof(USHORT);            
        regAddr = INDEX_PREF_BRIGHTNESS;
        value /= STEPPING_DELTA_BRIGHTNESS; // map to camera 
        break;
        
    default:
        TRAP();
        status = STATUS_NOT_SUPPORTED;
    }

    if (NT_SUCCESS(status)) {
        status = USBCAMD_ControlVendorCommand(DeviceContext,
                                              REQUEST_SET_PREFERENCE,
                                              (USHORT)value,
                                              regAddr,
                                              NULL,
                                              &length,
                                              FALSE,
                                              NULL,
                                              NULL);
    }

    INTELCAM_KdPrint(MIN_TRACE, ("'Setting property <0x%x>\n", status));

    Srb->Status = status;

    return status;
}


/*
** INTELCAM_GetCustomProperty()
**
** Arguments:
**
**  DeviceContext - driver context
**
** Returns:
**
**  NT status completion code 
**  
** Side Effects:  none
*/

NTSTATUS
INTELCAM_GetCustomProperty(
    PINTELCAM_DEVICE_CONTEXT DeviceContext,
    PHW_STREAM_REQUEST_BLOCK pSrb
    )
{
    NTSTATUS ntStatus = STATUS_NOT_SUPPORTED;
    
    return ntStatus;
}



/*
** INTELCAM_SetCustomProperty()
**
** Arguments:
**
**  DeviceContext - driver context
**
** Returns:
**
**  NT status completion code 
**  
** Side Effects:  none
*/

NTSTATUS
INTELCAM_SetCustomProperty(
    PINTELCAM_DEVICE_CONTEXT pDC,
    PHW_STREAM_REQUEST_BLOCK pSrb
    )
{
    NTSTATUS ntStatus = STATUS_NOT_SUPPORTED;
    
    return ntStatus;	
}

