#ifndef __PRPOBJ_H__
#define __PRPOBJ_H__

#if defined( mmioFOURCC )
    #undef mmioFOURCC
#endif // mmioFOURCC

#include "windef.h"
#include "mmsystem.h"



//
// for set/get props
//
#define CURRENT_VALUE 0
#define DEFAULT_VALUE 4



////////////////
/////
/////  The following are IRU Units wherever possible.  A mapping between
/////  these and the camera properties will be defined later in this file.
/////  The values are however based on the camera.
/////
////////////////
#define MAX_BRIGHTNESS_IRE_UNITS   119      // approx. measured value on UCAM1
#define STEPS_BRIGHTNESS 10 
#define STEPPING_DELTA_BRIGHTNESS    \
           ((MAX_BRIGHTNESS_IRE_UNITS+1)/STEPS_BRIGHTNESS)


#define MS_PROPERTYSET_INDEX    0
#define MANF_PROPERTYSET_INDEX  1

//
//  Custom properties
//
typedef enum _REQUEST_CUSTOM {
    FIRMWARE_VERSION = 3,
} REQUEST_CUSTOM;


#define MANF_NUMBEROFPROPERTIES 11


extern GUID PROPSETID_INTEL_CUSTOM_PROP;

typedef enum {
    KSPROPERTY_CUSTOM_PROP_FIRMWARE_VER            // RW O
} KSPROPERTY_CUSTOM_PROP;


typedef struct {
    KSPROPERTY Property;
    ULONG  Instance;                    // 0 for first PropAmp, 1 for 2nd, ...
    LONG   Value;                       // Value to set or get
    ULONG  Flags;                       // KSPROPERTY_CUSTOM_PROP_FLAGS_*
    ULONG  Capabilities;                // KSPROPERTY_CUSTOM_PROP_FLAGS_*
} KSPROPERTY_CUSTOM_PROP_S, *PKSPROPERTY_CUSTOM_PROP_S;

#define KSPROPERTY_CUSTOM_PROP_FLAGS_MANUAL      0X0000L
#define KSPROPERTY_CUSTOM_PROP_FLAGS_AUTO        0X0001L
#define KSPROPERTY_CUSTOM_PROP_FLAGS_DEFAULT     0X0004L


//
// Macros for packing/unpacking pan/scale.
//
#define MakeULong(l,h) ((unsigned long)(((unsigned long)(l)) | ((unsigned long)((unsigned short)(h))) << 16))
#define LoUShort(r) ((unsigned short)(r))
#define HiUShort(r) ((unsigned short)(((unsigned long)(r)) >> 16))
#define PackUShortAsULong(us,ul) \
    ul = us >> 8; \
    ul <<= 16; \
    ul |= (unsigned char)us
#define PackULongAsUShort(ul,us) \
    us = (unsigned short)ul; \
    us |= (unsigned short)(ul >> 8)

//
// Format four character codes.
//
#define FCC_FORMAT_YUV12N mmioFOURCC('I','Y','U','V')

//
// Format.
//
typedef struct _FORMAT {
    FOURCC Fcc;
    LONG lWidth;
    LONG lHeight;
} FORMAT;

//
// Generic range of control values.
//
typedef struct _RANGE {
    union {
        LONG Min;
        LONG NegX;
    };
    union {
        LONG Max;
        LONG PosX;
    };
    union {
        LONG PMin;
        LONG NegY;
    };
    union {
        LONG PMax;
        LONG PosY;
    };
    ULONG Step;
    LONG Default;
} RANGE;

//
// Control request IDs.
//
typedef enum _REQUEST {
    REQ_BRIGHTNESS      = 1,
} REQUEST;

//
// (Usec) Frame rate macro.
//
#define MICROSEC_PERFRAME(x)    ((x > 0) ? (ULONG)(1000000/x) : 0)

#define DRIVER_DATA_DONTCARE 0xffffffff

extern const RANGE _Brightness;

#endif /* __PRPOBJ_H__ */
